@@ -308,13 +308,8 @@ KateExternalToolsMenuAction::KateExternalToolsMenuAction( const QString &text,
 
 void KateExternalToolsMenuAction::reload()
 {
-
-  uint i = 0;
-  while ( i < m_actionCollection->count() )
-  {
-    m_actionCollection->action( i++ )->setShortcut(KShortcut());
-  }
   m_actionCollection->clear ();
+  popupMenu()->clear();
 
   // load all the tools, and create a action for each of them
   KConfig *config = new KConfig( "externaltools", false, false, "appdata" );
@@ -552,9 +547,9 @@ void KateExternalToolServiceEditor::showMTDlg()
 {
   QString text = i18n("Select the MimeTypes for which to enable this tool.");
   QStringList list = QStringList::split( QRegExp("\\s*;\\s*"), leMimetypes->text() );
-  KMimeTypeChooserDialog *d = new KMimeTypeChooserDialog( i18n("Select Mime Types"), text, list, "text", this );
-  if ( d->exec() == KDialogBase::Accepted ) {
-    leMimetypes->setText( d->chooser()->mimeTypes().join(";") );
+  KMimeTypeChooserDialog d( i18n("Select Mime Types"), text, list, "text", this );
+  if ( d.exec() == KDialogBase::Accepted ) {
+    leMimetypes->setText( d.chooser()->mimeTypes().join(";") );
   }
 }
 //END KateExternalToolServiceEditor
@@ -709,10 +704,13 @@ void KateExternalToolsConfigWidget::apply()
     // clean up the list of removed items, so that it does not contain
     // non-existing groups (we can't remove groups from a non-owned global file).
     config->sync();
-    for ( QStringList::iterator it1 = removed.begin(); it1 != removed.end(); ++it1 )
+    QStringList::iterator it1 = removed.begin();
+    while( it1 != removed.end() )
     {
       if ( ! config->hasGroup( *it1 ) )
-        removed.remove( *it1 );
+        it1 = removed.remove( it1 );
+      else
+        ++it1;
     }
     config->writeEntry( "removed", removed );
   }
@@ -724,7 +722,7 @@ void KateExternalToolsConfigWidget::slotSelectionChanged()
 {
   // update button state
   bool hs =  lbTools->selectedItem() != 0;
-  btnEdit->setEnabled( hs && static_cast<ToolItem*>(lbTools->selectedItem()) );
+  btnEdit->setEnabled( hs && dynamic_cast<ToolItem*>(lbTools->selectedItem()) );
   btnRemove->setEnabled( hs );
   btnMoveUp->setEnabled( ( lbTools->currentItem() > 0 ) && hs );
   btnMoveDwn->setEnabled( ( lbTools->currentItem() < (int)lbTools->count()-1 )&&hs );
@@ -773,6 +771,7 @@ void KateExternalToolsConfigWidget::slotRemove()
 
 void KateExternalToolsConfigWidget::slotEdit()
 {
+  if( !dynamic_cast<ToolItem*>(lbTools->selectedItem()) ) return;
   // show the item in an editor
   KateExternalTool *t = ((ToolItem*)lbTools->selectedItem())->tool;
   KateExternalToolServiceEditor editor( t, this);
