@@ -173,7 +173,7 @@ void KWrite::setupActions()
   a=KStdAction::keyBindings(this, SLOT(editKeys()), actionCollection());
   a->setWhatsThis(i18n("Configure the application's keyboard shortcut assignments."));
 
-  a=KStdAction::configureToolbars(this, SLOT(editToolbars()), actionCollection(), "set_configure_toolbars");
+  a=KStdAction::configureToolbars(this, SLOT(editToolbars()), actionCollection());
   a->setWhatsThis(i18n("Configure which items should appear in the toolbar(s)."));
 }
 
@@ -297,23 +297,19 @@ void KWrite::editKeys()
 
 void KWrite::editToolbars()
 {
+  saveMainWindowSettings( kapp->config(), "MainWindow" );
   KEditToolbar *dlg = new KEditToolbar(guiFactory());
-
-  if (dlg->exec())
-  {
-    KParts::GUIActivateEvent ev1( false );
-    QApplication::sendEvent( m_view, &ev1 );
-    guiFactory()->removeClient( m_view );
-    createShellGUI( false );
-    createShellGUI( true );
-    guiFactory()->addClient( m_view );
-    KParts::GUIActivateEvent ev2( true );
-    QApplication::sendEvent( m_view, &ev2 );
-  }
-
+  connect( dlg, SIGNAL(newToolbarConfig()), this, SLOT(slotNewToolbarConfig()) );
+  dlg->exec();
   delete dlg;
 }
 
+void KWrite::slotNewToolbarConfig()
+{
+  applyMainWindowSettings( kapp->config(), "MainWindow" );
+}
+
+
 void KWrite::printNow()
 {
   KTextEditor::printInterface(m_view->document())->print ();
@@ -550,7 +546,7 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
   QString kWriteVersion  = QString ("%1.%2.%3").arg(KDE::versionMajor() + 1).arg(KDE::versionMinor()).arg(KDE::versionRelease());
 
   KAboutData aboutData ( "kwrite",
-                         I18N_NOOP("KWrite"), 
+                         I18N_NOOP("KWrite"),
                          kWriteVersion.latin1(),
                          I18N_NOOP( "KWrite - Text Editor" ), KAboutData::License_LGPL_V2,
                          I18N_NOOP( "(c) 2000-2005 The Kate Authors" ), 0, "http://kate.kde.org" );
@@ -713,3 +709,4 @@ void KWriteEditorChooser::slotOk() {
 	m_chooser->writeAppSetting();
 	KDialogBase::slotOk();
 }
+// kate: space-indent on; indent-width 2; replace-tabs on; mixed-indent off;
