@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
     Permission is also granted to link this program with the Qt
     library, treating Qt like a library that normally accompanies the
@@ -25,38 +25,37 @@
 
 #include <unistd.h>
 
+#include <qcombobox.h>
+#include <qdir.h>
 #include <qlayout.h>
 #include <qpushbutton.h>
-#include <qcombobox.h>
-#include <qslider.h>
-#include <qdir.h>
-#include <qwhatsthis.h>
 #include <qregexp.h>
+#include <qslider.h>
 #include <qtabwidget.h>
+#include <qwhatsthis.h>
 
 #include <dcopref.h>
 
-#include <kapplication.h>
 #include <kaboutdata.h>
-#include <ksimpleconfig.h>
-#include <kmessagebox.h>
-#include <kstandarddirs.h>
-#include <kprocess.h>
+#include <kapplication.h>
+#include <kcmoduleloader.h>
+#include <kdebug.h>
 #include <kdialog.h>
+#include <klineedit.h>
+#include <kmessagebox.h>
+#include <kprocess.h>
+#include <krichtextlabel.h>
+#include <ksimpleconfig.h>
+#include <kstandarddirs.h>
 #include <kurlrequester.h>
 #include <libkmid/deviceman.h>
-#include <kdebug.h>
-#include <kcmoduleloader.h>
-#include <klineedit.h>
 
-//#include <kparts/componentfactory.h>
-//#include "midi.h"
 #include "arts.h"
 
 extern "C" {
-	void init_arts();
+	KDE_EXPORT void init_arts();
 
-    KCModule *create_arts(QWidget *parent, const char */*name*/)
+    KDE_EXPORT KCModule *create_arts(QWidget *parent, const char* /*name*/)
 	{
 		KGlobal::locale()->insertCatalogue("kcmarts");
 		return new KArtsModule(parent, "kcmarts" );
@@ -144,6 +143,12 @@ KArtsModule::KArtsModule(QWidget *parent, const char *name)
 {
 	setButtons(Default|Apply);
 
+	setQuickHelp( i18n("<h1>Sound System</h1> Here you can configure aRts, KDE's sound server."
+		    " This program not only allows you to hear your system sounds while simultaneously"
+		    " listening to an MP3 file or playing a game with background music. It also allows you"
+		    " to apply different effects to your system sounds and provides programmers with"
+		    " an easy way to achieve sound support."));
+
 	initAudioIOList();
 
 	QVBoxLayout *layout = new QVBoxLayout(this, 0, KDialog::spacingHint());
@@ -157,6 +162,7 @@ KArtsModule::KArtsModule(QWidget *parent, const char *name)
 
 	general->layout()->setMargin( KDialog::marginHint() );
 	hardware->layout()->setMargin( KDialog::marginHint() );
+	general->latencyLabel->setFixedHeight(general->latencyLabel->fontMetrics().lineSpacing());
 
 	tab->addTab(general, i18n("&General"));
 	tab->addTab(hardware, i18n("&Hardware"));
@@ -206,7 +212,7 @@ KArtsModule::KArtsModule(QWidget *parent, const char *name)
 	};
 
 	config = new KConfig("kcmartsrc");
-	GetSettings();
+	load();
 
 	suspendTime->setRange( 0, 999, 1, true );
 
@@ -230,7 +236,6 @@ KArtsModule::KArtsModule(QWidget *parent, const char *name)
 	connect(autoSuspend,SIGNAL(clicked()),SLOT(slotChanged()));
 	connect(suspendTime,SIGNAL(valueChanged(int)),SLOT(slotChanged()));
 	connect(general->testSound,SIGNAL(clicked()),SLOT(slotTestSound()));
-	connect(general->testMIDI,SIGNAL(clicked()),SLOT(slotTestMIDI()));
 	connect(hardware->midiDevice, SIGNAL( highlighted(int) ), this, SLOT( slotChanged() ) );
 	connect(hardware->midiDevice, SIGNAL( activated(int) ), this, SLOT( slotChanged() ) );
 	connect(hardware->midiUseMapper, SIGNAL( clicked() ), this, SLOT( slotChanged() ) );
@@ -245,8 +250,9 @@ KArtsModule::KArtsModule(QWidget *parent, const char *name)
 	setAboutData(about);
 }
 
-void KArtsModule::GetSettings( void )
+void KArtsModule::load( bool useDefaults )
 {
+   config->setReadDefaults( useDefaults );
 	config->setGroup("Arts");
 	startServer->setChecked(config->readBoolEntry("StartServer",true));
 	startRealtime->setChecked(config->readBoolEntry("StartRealtime",true) &&
@@ -312,7 +318,7 @@ void KArtsModule::GetSettings( void )
 	delete midiConfig;
 
 	updateWidgets();
-	emit changed( false );
+   emit changed( useDefaults );
 }
 
 KArtsModule::~KArtsModule() {
@@ -384,7 +390,7 @@ void KArtsModule::saveParams( void )
 
 void KArtsModule::load()
 {
-	GetSettings();
+   load( false );
 }
 
 void KArtsModule::save()
@@ -409,7 +415,7 @@ int KArtsModule::userSavedChanges()
                             "you restarted the sound server.\n"
                             "Do you want to save them?");
 	QString caption = i18n("Save Sound Server Settings?");
-	reply = KMessageBox::questionYesNo(this, question, caption);
+	reply = KMessageBox::questionYesNo(this, question, caption,KStdGuiItem::save(),KStdGuiItem::discard());
 	if ( reply == KMessageBox::Yes)
 	{
         configChanged = false;
@@ -430,49 +436,9 @@ void KArtsModule::slotTestSound()
 	test.start(KProcess::DontCare);
 }
 
-void KArtsModule::slotTestMIDI()
-{
-	if (configChanged && (userSavedChanges() == KMessageBox::Yes) || !artsdIsRunning() )
-		restartServer();
-
-	//KProcess test;
-	//test << "artsplay";
-	//test << locate("sound", "KDE_Startup_1.ogg");
-	//test.start(KProcess::DontCare);
-}
-
-
 void KArtsModule::defaults()
 {
-	startServer->setChecked(true);
-	startRealtime->setChecked(startRealtime->isEnabled() &&
-	                          realtimeIsPossible());
-	networkTransparent->setChecked(false);
-	fullDuplex->setChecked(false);
-	autoSuspend->setChecked(true);
-	suspendTime->setValue(60);
-	customDevice->setChecked(false);
-	deviceName->setText(QString::null);
-	customRate->setChecked(false);
-	samplingRate->setValue(44100);
-	general->latencySlider->setValue(250);
-	hardware->customOptions->setChecked(false);
-	hardware->addOptions->setText(QString::null);
-	hardware->audioIO->setCurrentItem(0);
-	hardware->soundQuality->setCurrentItem(0);
-	hardware->midiUseMapper->setChecked(false);
-	hardware->midiMapper->lineEdit()->clear();
-
-	slotChanged();
-}
-
-QString KArtsModule::quickHelp() const
-{
-	return i18n("<h1>Sound System</h1> Here you can configure aRts, KDE's sound server."
-		    " This program not only allows you to hear your system sounds while simultaneously"
-		    " listening to an MP3 file or playing a game with background music. It also allows you"
-		    " to apply different effects to your system sounds and provides programmers with"
-		    " an easy way to achieve sound support.");
+   load( true );
 }
 
 void KArtsModule::calculateLatency()
@@ -542,7 +508,6 @@ void KArtsModule::updateWidgets()
 	calculateLatency();
 
 	general->testSound->setEnabled(startServerIsChecked);
-	general->testMIDI->setEnabled(startServerIsChecked);
 
 //	general->volumeSystray->setEnabled(startServerIsChecked);
 	general->networkedSoundGroupBox->setEnabled(startServerIsChecked);
@@ -591,38 +556,31 @@ bool KArtsModule::realtimeIsPossible()
 	return realtimePossible;
 }
 
-void KArtsModule::initServer()
+void KArtsModule::restartServer()
 {
-	if (!startArts())
-		return;
+	config->setGroup("Arts");
+        bool starting = config->readBoolEntry("StartServer", true);
+	bool restarting = artsdIsRunning();
 
-	int count = 20;
-	int result = 0;
-	do {
-		sleep(1);
-	}
-	while (!artsdIsRunning() && count--);
-}
+	// Shut down knotify
+	DCOPRef("knotify", "qt/knotify").send("quit");
 
-void KArtsModule::stopServer()
-{
+	// Shut down artsd
 	KProcess terminateArts;
 	terminateArts << "artsshell";
 	terminateArts << "terminate";
 	terminateArts.start(KProcess::Block);
 
-	// leave artsd some time to go away
-	// FIXME(gioele): loop until artsshell status returns ok
-	//                and add a KProgressDialog
+	if (starting)
+	{
+		// Wait for artsd to shutdown completely and then (re)start artsd again
+		KStartArtsProgressDialog dlg(this, "start_arts_progress",
+	                       restarting ? i18n("Restarting Sound System") : i18n("Starting Sound System"),
+	                       restarting ? i18n("Restarting sound system.") : i18n("Starting sound system."));
+	        dlg.exec();
+	}
 
-	sleep(1);
-}
-
-void KArtsModule::restartServer()
-{
-	DCOPRef("knotify", "qt/knotify").send("quit");
-	stopServer();
-	initServer();
+	// Restart knotify
 	kapp->startServiceByDesktopName("knotify");
 }
 
@@ -637,7 +595,6 @@ bool KArtsModule::artsdIsRunning()
 }
 
 
-
 void init_arts()
 {
 	startArts();
@@ -692,6 +649,60 @@ QString KArtsModule::createArgs(bool netTrans,
 	return args;
 }
 
+KStartArtsProgressDialog::KStartArtsProgressDialog(KArtsModule *parent, const char *name,
+                          const QString &caption, const QString &text)
+ : KProgressDialog(parent, name, caption, text, true), m_module(parent), m_shutdown(false)
+{
+  connect(&m_timer, SIGNAL(timeout()), this, SLOT(slotProgress()));
+  progressBar()->setTotalSteps(20);
+  m_timeStep = 700;
+  m_timer.start(m_timeStep);
+  setAutoClose(false);
+}
+
+void
+KStartArtsProgressDialog::slotProgress()
+{
+  int p = progressBar()->progress();
+  if (p == 18)
+  {
+     progressBar()->reset();
+     progressBar()->setProgress(1);
+     m_timeStep = m_timeStep * 2;
+     m_timer.start(m_timeStep);
+  }
+  else
+  {
+     progressBar()->setProgress(p+1);
+  }
+
+  if (!m_shutdown)
+  {
+     // Wait for arts to shutdown
+     if (!m_module->artsdIsRunning())
+     {
+     	// Shutdown complete, restart
+     	if (!startArts())
+     		slotFinished(); // Strange, it didn't start
+     	else
+	     	m_shutdown = true;
+     }
+  }
+  
+  // Shut down completed? Wait for artsd to come up again
+  if (m_shutdown && m_module->artsdIsRunning())
+     slotFinished(); // Restart complete
+}
+
+void
+KStartArtsProgressDialog::slotFinished()
+{
+  progressBar()->setProgress(20);
+  m_timer.stop();
+  QTimer::singleShot(1000, this, SLOT(close()));
+}
+
+
 #ifdef I18N_ONLY
 	//lukas: these are hacks to allow translation of the following
 	I18N_NOOP("No Audio Input/Output");
