@@ -57,6 +57,7 @@ public:
   void saveParams( void );
 
   void load();
+  void load( bool useDefaults );
   void save();
   void defaults();
 
@@ -66,7 +67,6 @@ private slots:
 
   void slotChanged();
   void slotTestSound();
-  void slotTestMIDI();
   void slotArtsdExited(KProcess* proc);
   void slotProcessArtsdOutput(KProcess* p, char* buf, int len);
   //void slotStartServerChanged();
@@ -81,7 +81,6 @@ private:
                      int rate, int bits, const QString &audioIO,
                      const QString &addOptions, bool autoSuspend,
                      int suspendTime);
-  void GetSettings ();
   int userSavedChanges();
 
   QCheckBox *startServer, *startRealtime, *networkTransparent,
