@@ -34,6 +34,7 @@
 #include <kprotocolinfo.h>
 #include <kstandarddirs.h>
 #include <kconfig.h>
+#include <kmimetype.h>
 
 #include "kshorturifilter.h"
 
@@ -70,6 +71,15 @@ static bool isValidShortURL( const QString& cmd, bool verbose = false )
     if (verbose)
       kdDebug() << "KShortURIFilter::isValidShortURL: " << cmd
                 << " matches FQDN_PATTERN" << endl;
+
+    // bug 133687
+#if 0 
+    // stuff like wallpaper.png matches the FQDN_PATTERN but is most
+    // likely not a domain
+    if (KMimeType::findByPath(cmd, 0, true /* fast mode */) != KMimeType::defaultMimeTypePtr())
+        return false;
+#endif
+
     return true;
   }
 
@@ -409,7 +419,7 @@ bool KShortURIFilter::filterURI( KURIFilterData& data ) const
   // line arguments or options that might have been supplied.
   QString exe = removeArgs( cmd );
   //kdDebug() << k_funcinfo << "findExe with " << exe << endl;
-  if( !KStandardDirs::findExe( exe ).isNull() && data.checkForExecutables() )
+  if( data.checkForExecutables() && !KStandardDirs::findExe( exe ).isNull() )
   {
     //kdDebug() << "EXECUTABLE  exe=" << exe << endl;
     setFilteredURI( data, KURL( exe ));
@@ -425,7 +435,7 @@ bool KShortURIFilter::filterURI( KURIFilterData& data ) const
   // be slow things down...
   if ( !isMalformed && !isLocalFullPath )
   {
-    QStringList protocols = KProtocolInfo::protocols();
+    const QStringList protocols = KProtocolInfo::protocols();
     for( QStringList::ConstIterator it = protocols.begin(); it != protocols.end(); ++it )
     {
       if( (url.protocol() == *it) )
