@@ -85,7 +85,7 @@ void KBackground::makeReadOnly()
 void KBackground::load()
 {
     m_pCBEnable->setChecked( config->readBoolEntry( "UseBackground", true ) );
-    m_background->load();
+    m_background->load(false);
     slotEnableChanged();
     emit changed(false);
 }
