@@ -207,7 +207,7 @@ void KPluginOptions::load( bool useDefaults )
   priority->setValue(100 - KCLAMP(config->readNumEntry("Nice Level", 0), 0, 19) * 5);
   updatePLabel(priority->value());
 
-  dirLoad( config );
+  dirLoad( config, useDefaults );
   pluginLoad( config );
 
   delete config;
@@ -364,18 +364,19 @@ void KPluginOptions::dirInit()
 }
 
 
-void KPluginOptions::dirLoad( KConfig *config )
+void KPluginOptions::dirLoad( KConfig *config, bool useDefault )
 {
     QStringList paths;
 
     // read search paths
 
     config->setGroup("Misc");
-    if ( config->hasKey( "scanPaths" ) )
+    if ( config->hasKey( "scanPaths" ) && !useDefault )
         paths = config->readListEntry( "scanPaths" );
     else {//keep sync with kdebase/nsplugins/pluginscan
         paths.append("$HOME/.mozilla/plugins");
         paths.append("$HOME/.netscape/plugins");
+        paths.append("/usr/lib/firefox/plugins");
         paths.append("/usr/lib64/browser-plugins");
         paths.append("/usr/lib/browser-plugins");
         paths.append("/usr/local/netscape/plugins");
@@ -390,6 +391,7 @@ void KPluginOptions::dirLoad( KConfig *config )
 	paths.append("/usr/lib64/netscape/plugins");
 	paths.append("/usr/lib64/mozilla/plugins");
         paths.append("$MOZILLA_HOME/plugins");
+
     }
 
     // fill list
