@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -69,28 +69,17 @@
 
 /**** DLL Interface for kcontrol ****/
 
-void applyMultiHead(bool active)
-{
-	// Pass env. var to kdeinit.
-	QCString name = "KDE_MULTIHEAD";
-	QCString value = active ? "true" : "false";
-	QByteArray params;
-	QDataStream stream(params, IO_WriteOnly);
-	stream << name << value;
-	kapp->dcopClient()->send("klauncher", "klauncher", "setLaunchEnv(QCString,QCString)", params);
-}
-
 // Plugin Interface
 // Danimo: Why do we use the old interface?!
 extern "C"
 {
-    KCModule *create_style(QWidget *parent, const char*)
+    KDE_EXPORT KCModule *create_style(QWidget *parent, const char*)
     {
         KGlobal::locale()->insertCatalogue("kcmstyle");
         return new KCMStyle(parent, "kcmstyle");
     }
 
-    void init_style()
+    KDE_EXPORT void init_style()
     {
         uint flags = KRdbExportQtSettings | KRdbExportQtColors | KRdbExportXftSettings;
         KConfig config("kcmdisplayrc", true /*readonly*/, false /*don't read kdeglobals etc.*/);
@@ -102,11 +91,6 @@ extern "C"
             flags |= KRdbExportColors;
         runRdb( flags );
 
-        // This key has no GUI apparently
-        bool isActive = !config.readBoolEntry( "disableMultihead", false) &&
-                        (ScreenCount(qt_xdisplay()) > 1);
-        applyMultiHead( isActive );
-
         // Write some Qt root property.
 #ifndef __osf__      // this crashes under Tru64 randomly -- will fix later
         QByteArray properties;
@@ -134,6 +118,11 @@ K_EXPORT_COMPONENT_FACTORY( kcm_kcmstyle, GeneralFactory )
 KCMStyle::KCMStyle( QWidget* parent, const char* name )
 	: KCModule( parent, name ), appliedStyle(NULL)
 {
+    setQuickHelp( i18n("<h1>Style</h1>"
+			"This module allows you to modify the visual appearance "
+			"of user interface elements, such as the widget style "
+			"and effects."));
+
 	m_bEffectsDirty = false;
 	m_bStyleDirty= false;
 	m_bToolbarsDirty = false;
@@ -216,10 +205,8 @@ KCMStyle::KCMStyle( QWidget* parent, const char* name )
 
 	// Add Page2 (Effects)
 	// -------------------
-	gbEffects = new QGroupBox( 1, Qt::Horizontal, i18n("GUI Effects"), page2 );
-	cbEnableEffects = new QCheckBox( i18n("&Enable GUI effects"), gbEffects );
-
-	containerFrame = new QFrame( gbEffects );
+	cbEnableEffects = new QCheckBox( i18n("&Enable GUI effects"), page2 );
+	containerFrame = new QFrame( page2 );
 	containerFrame->setFrameStyle( QFrame::NoFrame | QFrame::Plain );
 	containerFrame->setMargin(0);
 	containerLayout = new QGridLayout( containerFrame, 1, 1,	// rows, columns
@@ -269,11 +256,11 @@ KCMStyle::KCMStyle( QWidget* parent, const char* name )
 	containerLayout->addItem( comboSpacer, 1, 2 );
 
 	// Separator.
-	QFrame* hline = new QFrame ( gbEffects );
+	QFrame* hline = new QFrame ( page2 );
 	hline->setFrameStyle( QFrame::HLine | QFrame::Sunken );
 
 	// Now implement the Menu Transparency container.
-	menuContainer = new QFrame( gbEffects );
+	menuContainer = new QFrame( page2 );
 	menuContainer->setFrameStyle( QFrame::NoFrame | QFrame::Plain );
 	menuContainer->setMargin(0);
 	menuContainerLayout = new QGridLayout( menuContainer, 1, 1,    // rows, columns
@@ -317,7 +304,11 @@ KCMStyle::KCMStyle( QWidget* parent, const char* name )
 	menuContainerLayout->addMultiCellWidget( menuPreview, 0, 3, 1, 1 );
 
 	// Layout page2.
-	page2Layout->addWidget( gbEffects );
+	page2Layout->addWidget( cbEnableEffects );
+	page2Layout->addWidget( containerFrame );
+	page2Layout->addWidget( hline );
+	page2Layout->addWidget( menuContainer );
+
 	QSpacerItem* sp1 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
 	page2Layout->addItem( sp1 );
 
@@ -332,11 +323,11 @@ KCMStyle::KCMStyle( QWidget* parent, const char* name )
 
 	// Add Page3 (Miscellaneous)
 	// -------------------------
-	gbToolbarSettings = new QGroupBox( 1, Qt::Horizontal, i18n("Toolbar Settings"), page3 );
-	cbHoverButtons = new QCheckBox( i18n("High&light buttons under mouse"), gbToolbarSettings );
-	cbTransparentToolbars = new QCheckBox( i18n("Transparent tool&bars when moving"), gbToolbarSettings );
+	cbHoverButtons = new QCheckBox( i18n("High&light buttons under mouse"), page3 );
+	cbTransparentToolbars = new QCheckBox( i18n("Transparent tool&bars when moving"), page3 );
 
-	QWidget * dummy = new QWidget( gbToolbarSettings );
+	QWidget * dummy = new QWidget( page3 );
+	
 	QHBoxLayout* box2 = new QHBoxLayout( dummy, 0, KDialog::spacingHint() );
 	lbl = new QLabel( i18n("Text pos&ition:"), dummy );
 	comboToolbarIcons = new QComboBox( FALSE, dummy );
@@ -350,9 +341,12 @@ KCMStyle::KCMStyle( QWidget* parent, const char* name )
 	box2->addWidget( comboToolbarIcons );
 	QSpacerItem* sp2 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
 	box2->addItem( sp2 );
-
+	
+	page3Layout->addWidget( cbHoverButtons );
+	page3Layout->addWidget( cbTransparentToolbars );
+	page3Layout->addWidget( dummy );
+	
 	// Layout page3.
-	page3Layout->addWidget( gbToolbarSettings );
 	QSpacerItem* sp3 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
 	page3Layout->addItem( sp3 );
 
@@ -384,7 +378,7 @@ KCMStyle::KCMStyle( QWidget* parent, const char* name )
 	// Insert the pages into the tabWidget
 	tabWidget->insertTab( page1, i18n("&Style"));
 	tabWidget->insertTab( page2, i18n("&Effects"));
-	tabWidget->insertTab( page3, i18n("&Miscellaneous"));
+	tabWidget->insertTab( page3, i18n("&Toolbar"));
 
 	//Enable/disable the button for the initial style
 	updateConfigButton();
@@ -461,7 +455,7 @@ void KCMStyle::styleSpecificConfig()
 
 	if (dial->exec() == QDialog::Accepted  && dial->isDirty() ) {
 		// Force re-rendering of the preview, to apply settings
-		switchStyle(currentStyle());
+		switchStyle(currentStyle(), true);
 
 		//For now, ask all KDE apps to recreate their styles to apply the setitngs
 		KIPC::sendMessageAll(KIPC::StyleChanged);
@@ -474,8 +468,16 @@ void KCMStyle::styleSpecificConfig()
 }
 
 void KCMStyle::load()
+{
+   load( false );
+}
+
+void KCMStyle::load(bool useDefaults)
 {
 	KConfig config( "kdeglobals", true, false );
+
+	config.setReadDefaults( useDefaults );
+
 	// Page1 - Build up the Style ListBox
 	loadStyle( config );
 
@@ -489,7 +491,7 @@ void KCMStyle::load()
 	m_bStyleDirty= false;
 	m_bToolbarsDirty = false;
 
-	emit changed( false );
+	emit changed( useDefaults );
 }
 
 
@@ -665,53 +667,7 @@ bool KCMStyle::findStyle( const QString& str, int& combobox_item )
 
 void KCMStyle::defaults()
 {
-	// Select default style
-	int item = 0;
-	bool found;
-
-	found = findStyle( KStyle::defaultStyle(), item );
-	if (!found)
-		found = findStyle( "highcolor", item );
-	if (!found)
-		found = findStyle( "default", item );
-	if (!found)
-		found = findStyle( "windows", item );
-	if (!found)
-		found = findStyle( "platinum", item );
-	if (!found)
-		found = findStyle( "motif", item );
-
-	cbStyle->setCurrentItem( item );
-
-	m_bStyleDirty = true;
-	switchStyle( currentStyle() );	// make resets visible
-
-	// Effects..
-	cbEnableEffects->setChecked(false);
-	comboTooltipEffect->setCurrentItem(0);
-	comboComboEffect->setCurrentItem(0);
-	comboMenuEffect->setCurrentItem(0);
-	comboMenuHandle->setCurrentItem(0);
-	comboMenuEffectType->setCurrentItem(0);
-	slOpacity->setValue(90);
-	cbMenuShadow->setChecked(false);
-
-	// Miscellanous
-	cbHoverButtons->setChecked(true);
-	cbTransparentToolbars->setChecked(true);
-	cbEnableTooltips->setChecked(true);
-	comboToolbarIcons->setCurrentItem(0);
-	cbIconsOnButtons->setChecked(false);
-	cbTearOffHandles->setChecked(false);
-}
-
-
-QString KCMStyle::quickHelp() const
-{
-	return i18n("<h1>Style</h1>"
-			"This module allows you to modify the visual appearance "
-			"of user interface elements, such as the widget style "
-			"and effects.");
+   load( true );
 }
 
 void KCMStyle::setEffectsDirty()
@@ -850,8 +806,12 @@ void KCMStyle::styleChanged()
 }
 
 
-void KCMStyle::switchStyle(const QString& styleName)
+void KCMStyle::switchStyle(const QString& styleName, bool force)
 {
+	// Don't flicker the preview if the same style is chosen in the cb
+	if (!force && appliedStyle && appliedStyle->name() == styleName) 
+		return;
+         
 	// Create an instance of the new style...
 	QStyle* style = QStyleFactory::create(styleName);
 	if (!style)
@@ -1058,7 +1018,7 @@ void KCMStyle::addWhatsThis()
 							"without having to apply it to the whole desktop.") );
 
 	// Page2
-	QWhatsThis::add( gbEffects, i18n("This page allows you to enable various widget style effects. "
+	QWhatsThis::add( page2, i18n("This page allows you to enable various widget style effects. "
 							"For best performance, it is advisable to disable all effects.") );
 	QWhatsThis::add( cbEnableEffects, i18n( "If you check this box, you can select several effects "
 							"for different widgets like combo boxes, menus or tooltips.") );
@@ -1082,7 +1042,7 @@ void KCMStyle::addWhatsThis()
 	QWhatsThis::add( slOpacity, i18n("By adjusting this slider you can control the menu effect opacity.") );
 
 	// Page3
-	QWhatsThis::add( gbToolbarSettings, i18n("<b>Note:</b> that all widgets in this combobox "
+	QWhatsThis::add( page3, i18n("<b>Note:</b> that all widgets in this combobox "
 							"do not apply to Qt-only applications.") );
 	QWhatsThis::add( cbHoverButtons, i18n("If this option is selected, toolbar buttons will change "
 							"their color when the mouse cursor is moved over them." ) );
