@@ -87,21 +87,28 @@ KMiniPager::KMiniPager(const QString& configFile, Type type, int actions,
       m_contextMenu(0),
       m_settings( new PagerSettings(sharedConfig()) )
 {
-    m_settings->readConfig();
-    m_windows.setAutoDelete(true);
-    TaskManager::the()->trackGeometry();
-
-    m_group = new QButtonGroup(this);
-    m_group->setExclusive( true );
-
-    setFont( KGlobalSettings::taskbarFont() );
-
     int scnum = QApplication::desktop()->screenNumber(this);
     QRect desk = QApplication::desktop()->screenGeometry(scnum);
     if (desk.width() <= 800)
     {
-        m_settings->setPreview(false);
+        KConfigSkeleton::ItemBool* item = dynamic_cast<KConfigSkeleton::ItemBool*>(m_settings->findItem("Preview"));
+        if (item)
+        {
+            item->setDefaultValue(false);
+        }
     }
+    m_settings->readConfig();
+    m_windows.setAutoDelete(true);
+    if (m_settings->preview())
+    {
+        TaskManager::the()->trackGeometry();
+    }
+
+    m_group = new QButtonGroup(this);
+    m_group->setFrameStyle(QFrame::NoFrame);
+    m_group->setExclusive( true );
+
+    setFont( KGlobalSettings::taskbarFont() );
 
     m_kwin = new KWinModule(this);
     m_activeWindow = m_kwin->activeWindow();
@@ -116,9 +123,14 @@ KMiniPager::KMiniPager(const QString& configFile, Type type, int actions,
     desktopLayoutX = -1;
     desktopLayoutY = -1;
 
+    QSize s(m_kwin->numberOfViewports(m_kwin->currentDesktop()));
+    m_useViewports = s.width() * s.height() > 1;
+
     drawButtons();
 
     connect( m_kwin, SIGNAL( currentDesktopChanged(int)), SLOT( slotSetDesktop(int) ) );
+    connect( m_kwin, SIGNAL( currentDesktopViewportChanged(int, const QPoint&)),
+            SLOT(slotSetDesktopViewport(int, const QPoint&)));
     connect( m_kwin, SIGNAL( numberOfDesktopsChanged(int)), SLOT( slotSetDesktopCount(int) ) );
     connect( m_kwin, SIGNAL( activeWindowChanged(WId)), SLOT( slotActiveWindowChanged(WId) ) );
     connect( m_kwin, SIGNAL( windowAdded(WId) ), this, SLOT( slotWindowAdded(WId) ) );
@@ -150,23 +162,24 @@ KMiniPager::~KMiniPager()
 {
     KGlobal::locale()->removeCatalogue("kminipagerapplet");
     delete m_contextMenu;
+    delete m_settings;
 }
 
 void KMiniPager::slotBackgroundChanged(int desk)
 {
-    int numDesktops = m_kwin->numberOfDesktops();
+    unsigned numDesktops = m_kwin->numberOfDesktops();
     if (numDesktops != m_desktops.count())
     {
         slotSetDesktopCount(numDesktops);
     }
 
-    if (desk < 1 || desk > m_desktops.count())
+    if (desk < 1 || (unsigned) desk > m_desktops.count())
     {
         // should not happen, but better to be paranoid than crash
         return;
     }
 
-    m_desktops[desk - 1]->loadBgPixmap();
+    m_desktops[desk - 1]->backgroundChanged();
 }
 
 void KMiniPager::slotSetDesktop(int desktop)
@@ -176,7 +189,7 @@ void KMiniPager::slotSetDesktop(int desktop)
         slotSetDesktopCount( m_kwin->numberOfDesktops() );
     }
 
-    if (desktop != KWin::currentDesktop())
+    if (!m_useViewports && (desktop != KWin::currentDesktop()))
     {
         // this can happen when the user clicks on a desktop,
         // holds down the key combo to switch desktops, lets the
@@ -200,9 +213,29 @@ void KMiniPager::slotSetDesktop(int desktop)
     }
 }
 
+void KMiniPager::slotSetDesktopViewport(int desktop, const QPoint& viewport)
+{
+    // ###
+    Q_UNUSED(desktop);
+    QSize s(m_kwin->numberOfViewports(m_kwin->currentDesktop()));
+    slotSetDesktop((viewport.y()-1) * s.width() + viewport.x() );
+}
+
 void KMiniPager::slotButtonSelected( int desk )
 {
-    KWin::setCurrentDesktop( desk );
+    if (m_kwin->numberOfViewports(m_kwin->currentDesktop()).width() *
+        m_kwin->numberOfViewports(m_kwin->currentDesktop()).height() > 1)
+    {
+        QPoint p;
+
+        p.setX( (desk-1) * QApplication::desktop()->screenGeometry().width());
+        p.setY( 0 );
+
+        KWin::setCurrentDesktopViewport(m_kwin->currentDesktop(), p);
+    }
+    else
+        KWin::setCurrentDesktop( desk );
+
     slotSetDesktop( desk );
 }
 
@@ -213,7 +246,9 @@ int KMiniPager::widthForHeight(int h) const
         return width();
     }
 
-    int deskNum = m_kwin->numberOfDesktops();
+    int deskNum = m_kwin->numberOfDesktops() * m_kwin->numberOfViewports(0).width()
+        * m_kwin->numberOfViewports(0).height();
+
     int rowNum = m_settings->numberOfRows();
     if (rowNum == 0)
     {
@@ -267,7 +302,8 @@ int KMiniPager::heightForWidth(int w) const
         return height();
     }
 
-    int deskNum = m_kwin->numberOfDesktops();
+    int deskNum = m_kwin->numberOfDesktops() * m_kwin->numberOfViewports(0).width()
+        * m_kwin->numberOfViewports(0).height();
     int rowNum = m_settings->numberOfRows(); // actually these are columns now... oh well.
     if (rowNum == 0)
     {
@@ -314,11 +350,18 @@ void KMiniPager::updateDesktopLayout(int o, int x, int y)
       return;
     }
 
+    QCString kwin_name;
+    int screen_number = DefaultScreen(qt_xdisplay());
+    if (screen_number == 0)
+        kwin_name = "kwin";
+    else
+        kwin_name.sprintf("kwin-screen-%d", screen_number);
+
     QCString replyType;
     QByteArray data, replyData;
     QDataStream arg(data, IO_WriteOnly);
     arg << o << x << y;
-    if ( !(kapp->dcopClient()->call( "kwin", "KWinInterface", "setDesktopLayout(int, int, int)",
+    if ( !(kapp->dcopClient()->call( kwin_name, "KWinInterface", "setDesktopLayout(int, int, int)",
            data, replyType, replyData)))
     {
         kdDebug() << "KMiniPager: Call to KWinInterface::setDesktopLayout(int, int, int) failed" << endl;
@@ -393,6 +436,8 @@ void KMiniPager::wheelEvent( QWheelEvent* e )
 {
     int newDesk;
     int desktops = KWin::numberOfDesktops();
+    if (m_kwin->numberOfViewports(0).width() * m_kwin->numberOfViewports(0).height() > 1 )
+        desktops = m_kwin->numberOfViewports(0).width() * m_kwin->numberOfViewports(0).height();
     if (e->delta() < 0)
     {
         newDesk = m_curDesk % desktops + 1;
@@ -410,26 +455,34 @@ void KMiniPager::drawButtons()
     int deskNum = m_kwin->numberOfDesktops();
     KMiniPagerButton *desk;
 
-    for ( int i = 1; i <= deskNum; ++i )
+    int count = 1;
+    int i = 1;
+    do
     {
-        desk = new KMiniPagerButton( i, this );
-        //btn->setOn(i == act);
-
-        if ( m_settings->labelType() != PagerSettings::EnumLabelType::LabelName )
+        QSize viewportNum = m_kwin->numberOfViewports(i);
+        for (int j = 1; j <= viewportNum.width() * viewportNum.height(); ++j)
         {
-            QToolTip::add( desk, desk->desktopName() );
+            QSize s(m_kwin->numberOfViewports(m_kwin->currentDesktop()));
+            QPoint viewport( (j-1) % s.width(), (j-1) / s.width());
+            desk = new KMiniPagerButton( count, m_useViewports, viewport, this );
+            if ( m_settings->labelType() != PagerSettings::EnumLabelType::LabelName )
+            {
+                QToolTip::add( desk, desk->desktopName() );
+            }
+
+            m_desktops.append( desk );
+            m_group->insert( desk, count );
+
+            connect(desk, SIGNAL(buttonSelected(int)),
+                    SLOT(slotButtonSelected(int)) );
+            connect(desk, SIGNAL(showMenu(const QPoint&, int )),
+                    SLOT(slotShowMenu(const QPoint&, int )) );
+
+            desk->show();
+            ++count;
         }
-
-        m_desktops.append( desk );
-        m_group->insert( desk, i );
-
-        connect(desk, SIGNAL(buttonSelected(int)),
-                SLOT(slotButtonSelected(int)) );
-        connect(desk, SIGNAL(showMenu(const QPoint&, int )),
-                SLOT(slotShowMenu(const QPoint&, int )) );
-
-        desk->show();
     }
+    while ( ++i <= deskNum );
 }
 
 void KMiniPager::slotSetDesktopCount( int )
@@ -466,8 +519,8 @@ void KMiniPager::slotActiveWindowChanged( WId win )
         QValueList<KMiniPagerButton*>::ConstIterator itEnd = m_desktops.end();
         for ( it = m_desktops.begin(); it != itEnd; ++it)
         {
-            if ( ( inf1 && ( inf1->onAllDesktops() || (inf1->desktop() == (*it)->desktop()) ) ) ||
-                 ( inf2 && ( inf2->onAllDesktops() || (inf2->desktop() == (*it)->desktop()) ) ) )
+            if ( ( inf1 && (*it)->shouldPaintWindow(inf1)) ||
+                 ( inf2 && (*it)->shouldPaintWindow(inf2)) )
             {
                 (*it)->windowsChanged();
             }
@@ -490,7 +543,7 @@ void KMiniPager::slotWindowAdded( WId win)
         QValueList<KMiniPagerButton*>::ConstIterator itEnd = m_desktops.end();
         for ( it = m_desktops.begin(); it != itEnd; ++it)
         {
-            if ( inf->onAllDesktops() || inf->desktop() == (*it)->desktop() )
+            if ( (*it)->shouldPaintWindow(inf) )
             {
                 (*it)->windowsChanged();
             }
@@ -504,6 +557,7 @@ void KMiniPager::slotWindowRemoved(WId win)
     {
         KWin::WindowInfo* inf = info(win);
         bool onAllDesktops = inf->onAllDesktops();
+        bool onAllViewports = inf->hasState(NET::Sticky);
         bool skipPager = inf->state() & NET::SkipPager;
         int desktop = inf->desktop();
 
@@ -521,7 +575,7 @@ void KMiniPager::slotWindowRemoved(WId win)
         QValueList<KMiniPagerButton*>::ConstIterator itEnd = m_desktops.end();
         for (it = m_desktops.begin(); it != itEnd; ++it)
         {
-            if (onAllDesktops || desktop == (*it)->desktop())
+            if (onAllDesktops || onAllViewports || desktop == (*it)->desktop())
             {
                 (*it)->windowsChanged();
             }
@@ -537,9 +591,9 @@ void KMiniPager::slotWindowRemoved(WId win)
 
 void KMiniPager::slotWindowChanged( WId win , unsigned int properties )
 {
-    if ((properties & (NET::WMState | NET::XAWMState |
-                       NET::WMDesktop | NET::WMGeometry)) == 0 &&
-        (desktopPreview() && windowIcons() &&
+    if ((properties & (NET::WMState | NET::XAWMState | NET::WMDesktop)) == 0 &&
+        (!desktopPreview() || (properties & NET::WMGeometry) == 0) &&
+        !(desktopPreview() && windowIcons() &&
          (properties & NET::WMIcon | NET::WMIconName | NET::WMVisibleIconName) == 0))
     {
         return;
@@ -548,22 +602,27 @@ void KMiniPager::slotWindowChanged( WId win , unsigned int properties )
     if (desktopPreview())
     {
         KWin::WindowInfo* inf = m_windows[win];
-        bool onAllDesktops = inf ? inf->onAllDesktops() : false;
-        bool skipPager = inf ? inf->state() & NET::SkipPager : false;
-        unsigned int desktop = inf ? inf->desktop() : 0;
-        m_windows.remove((long) win);
+        bool skipPager = inf->hasState(NET::SkipPager);
+        QMemArray<bool> old_shouldPaintWindow(m_desktops.size());
+        QValueList<KMiniPagerButton*>::ConstIterator it;
+        QValueList<KMiniPagerButton*>::ConstIterator itEnd = m_desktops.end();
+        int i = 0;
+        for ( it = m_desktops.begin(); it != itEnd; ++it)
+        {
+            old_shouldPaintWindow[i++] = (*it)->shouldPaintWindow(inf);
+        }
+
+        m_windows.remove(win);
         inf = info(win);
 
-        if (inf->state() & NET::SkipPager || skipPager)
+        if (inf->hasState(NET::SkipPager) || skipPager)
         {
             return;
         }
 
-        QValueList<KMiniPagerButton*>::ConstIterator it;
-        QValueList<KMiniPagerButton*>::ConstIterator itEnd = m_desktops.end();
-        for ( it = m_desktops.begin(); it != itEnd; ++it)
+        for ( i = 0, it = m_desktops.begin(); it != itEnd; ++it)
         {
-            if ( inf->onAllDesktops() || inf->desktop() == (*it)->desktop() || onAllDesktops || desktop == (*it)->desktop() )
+            if ( old_shouldPaintWindow[i++] || (*it)->shouldPaintWindow(inf))
             {
                 (*it)->windowsChanged();
             }
@@ -578,12 +637,12 @@ void KMiniPager::slotWindowChanged( WId win , unsigned int properties )
 
 KWin::WindowInfo* KMiniPager::info( WId win )
 {
-    if (!m_windows[win] )
+    if (!m_windows[win])
     {
         KWin::WindowInfo* info = new KWin::WindowInfo( win,
             NET::WMWindowType | NET::WMState | NET::XAWMState | NET::WMDesktop | NET::WMGeometry | NET::WMKDEFrameStrut, 0 );
 
-        m_windows.insert( (long) win, info );
+        m_windows.insert(win, info);
         return info;
     }
 
@@ -722,6 +781,7 @@ void KMiniPager::contextMenuActivated(int result)
     {
         case WindowThumbnails:
             m_settings->setPreview(!m_settings->preview());
+            TaskManager::the()->trackGeometry();
             break;
 
         case WindowIcons:
@@ -735,8 +795,16 @@ void KMiniPager::contextMenuActivated(int result)
             m_settings->setBackgroundType(PagerSettings::EnumBackgroundType::BgTransparent);
             break;
         case PagerSettings::EnumBackgroundType::BgLive + bgOffset:
+        {
             m_settings->setBackgroundType(PagerSettings::EnumBackgroundType::BgLive);
+            QValueList<KMiniPagerButton*>::ConstIterator it;
+            QValueList<KMiniPagerButton*>::ConstIterator itEnd = m_desktops.end();
+            for( it = m_desktops.begin(); it != itEnd; ++it )
+            {
+                (*it)->backgroundChanged();
+            }
             break;
+        }
 
         case PagerSettings::EnumLabelType::LabelNone + labelOffset:
             m_settings->setLabelType(PagerSettings::EnumLabelType::LabelNone);
