@@ -20,6 +20,7 @@
 #include <klocale.h>
 #include <kde_file.h>
 #include <kio/global.h>
+#include <kremoteencoding.h>
 
 #include <errno.h> // to be removed
 
@@ -178,7 +179,7 @@ void ArchiveProtocol::createUDSEntry( const KArchiveEntry * archiveEntry, UDSEnt
     UDSAtom atom;
     entry.clear();
     atom.m_uds = UDS_NAME;
-    atom.m_str = archiveEntry->name();
+    atom.m_str = remoteEncoding()->decode(archiveEntry->name().local8Bit());
     entry.append(atom);
 
     atom.m_uds = UDS_FILE_TYPE;
@@ -198,15 +199,15 @@ void ArchiveProtocol::createUDSEntry( const KArchiveEntry * archiveEntry, UDSEnt
     entry.append( atom );
 
     atom.m_uds = UDS_USER;
-    atom.m_str = archiveEntry->user();
+    atom.m_str = remoteEncoding()->decode(archiveEntry->user().local8Bit());
     entry.append( atom );
 
     atom.m_uds = UDS_GROUP;
-    atom.m_str = archiveEntry->group();
+    atom.m_str = remoteEncoding()->decode(archiveEntry->group().local8Bit());
     entry.append( atom );
 
     atom.m_uds = UDS_LINK_DEST;
-    atom.m_str = archiveEntry->symlink();
+    atom.m_str = remoteEncoding()->decode(archiveEntry->symlink().local8Bit());
     entry.append( atom );
 }
 
@@ -256,6 +257,8 @@ void ArchiveProtocol::listDir( const KURL & url )
         return;
     }
 
+    path = QString::fromLocal8Bit(remoteEncoding()->encode(path));
+
     kdDebug( 7109 ) << "checkNewFile done" << endl;
     const KArchiveDirectory* root = m_archiveFile->directory();
     const KArchiveDirectory* dir;
@@ -360,6 +363,7 @@ void ArchiveProtocol::stat( const KURL & url )
         path = QString::fromLatin1( "/" );
         archiveEntry = root;
     } else {
+        path = QString::fromLocal8Bit(remoteEncoding()->encode(path));
         archiveEntry = root->entry( path );
     }
     if ( !archiveEntry )
@@ -399,6 +403,8 @@ void ArchiveProtocol::get( const KURL & url )
         }
     }
 
+    path = QString::fromLocal8Bit(remoteEncoding()->encode(path));
+
     const KArchiveDirectory* root = m_archiveFile->directory();
     const KArchiveEntry* archiveEntry = root->entry( path );
 
