@@ -52,7 +52,7 @@ ConfigDialog::ConfigDialog( const ActionList *list, KGlobalAccel *accel,
     w = addVBoxPage( i18n("Ac&tions") );
     actionWidget = new ActionWidget( list, this, w, "actions widget" );
 
-    w = addVBoxPage( i18n("&Shortcuts") );
+    w = addVBoxPage( i18n("Global &Shortcuts") );
     keysWidget = new KKeyChooser( accel, w );
 }
 
