@@ -13,7 +13,7 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
  */
 
@@ -21,26 +21,42 @@
 #define __kmenuedit_h__
 
 #include <kmainwindow.h>
+#include <treeview.h>
 
-class MenuEditView;
+class BasicTab;
+class QSplitter;
+class KAction;
+class KToggleAction;
 
 class KMenuEdit : public KMainWindow
 {
     Q_OBJECT
 
 public:
-    KMenuEdit( QWidget *parent=0, const char *name=0 );
+    KMenuEdit( bool controlCenter, QWidget *parent=0, const char *name=0 );
     ~KMenuEdit();
 
+    void selectMenu(const QString &menu) { m_tree->selectMenu(menu); }
+    void selectMenuEntry(const QString &menuEntry) { m_tree->selectMenuEntry(menuEntry); }
+
 protected:
     void setupView();
     void setupActions();
+    bool queryClose();
 
 protected slots:
-    void slotClose();
-
+    void slotSave();
+    void slotChangeView();
+    void slotConfigureToolbars();
 protected:
-    MenuEditView *_view;
+    TreeView           *m_tree;
+    BasicTab           *m_basicTab;
+    QSplitter          *m_splitter;
+
+    KAction *m_actionDelete;
+    KToggleAction *m_actionShowHidden;
+    bool m_showHidden;
+    bool m_controlCenter;
 };
 
 #endif
