@@ -56,6 +56,8 @@ KonqViewManager::KonqViewManager( KonqMainWindow *mainWindow )
   m_bProfileListDirty = true;
   m_bLoadingProfile = false;
 
+  m_activePartChangedTimer = new QTimer(this);
+  connect(m_activePartChangedTimer, SIGNAL(timeout()), this, SLOT(emitActivePartChanged()));
   connect( this, SIGNAL( activePartChanged ( KParts::Part * ) ), this, SLOT( slotActivePartChanged ( KParts::Part * ) ) );
 }
 
@@ -731,7 +733,7 @@ void KonqViewManager::updatePixmaps()
 
   tabContainer->listViews( &viewList );
   for ( it.toFirst(); it != 0L; ++it )
-    it.current()->setTabIcon( it.current()->locationBarURL() );
+    it.current()->setTabIcon( KURL::fromPathOrURL( it.current()->locationBarURL() ) );
 }
 
 void KonqViewManager::removeView( KonqView *view )
@@ -1198,7 +1200,8 @@ void KonqViewManager::loadViewProfile( KConfig &cfg, const QString & filename,
 
   QString rootItem = cfg.readEntry( "RootItem", "empty" );
 
-  //kdDebug(1202) << "KonqViewManager::loadViewProfile : loading RootItem " << rootItem << endl;
+  //kdDebug(1202) << "KonqViewManager::loadViewProfile : loading RootItem " << rootItem << 
+  //" forcedURL " << forcedURL.url() << endl;
 
   if ( forcedURL.url() != "about:blank" )
   {
@@ -1222,6 +1225,8 @@ void KonqViewManager::loadViewProfile( KConfig &cfg, const QString & filename,
     m_pMainWindow->action( "clear_location" )->activate();
   }
 
+  //kdDebug(1202) << "KonqViewManager::loadViewProfile : after loadItem " << endl;
+
   if (m_pDocContainer == 0L)
   {
     if (m_pMainWindow &&
@@ -1235,7 +1240,7 @@ void KonqViewManager::loadViewProfile( KConfig &cfg, const QString & filename,
   }
 
 
-    if ( m_pDocContainer->frameType() != "Tabs")
+  if ( m_pDocContainer->frameType() != "Tabs")
     convertDocContainer();
 
   static_cast<KonqFrameTabs*>( m_pDocContainer )->setAlwaysTabbedMode( alwaysTabbedMode );
@@ -1287,8 +1292,8 @@ void KonqViewManager::loadViewProfile( KConfig &cfg, const QString & filename,
          QSize size = readConfigSize( cfg, m_pMainWindow );
          if ( size.isValid() )
              m_pMainWindow->resize( size );
-         else if( resetWindow )
-             m_pMainWindow->resize( 700, 480 ); // size from KonqMainWindow ctor
+         else  // no size in the profile; use last known size
+             m_pMainWindow->restoreWindowSize();
      }
   }
 
@@ -1355,18 +1360,17 @@ void KonqViewManager::setActivePart( KParts::Part *part, bool immediate )
     if (part && part->widget())
         part->widget()->setFocus();
 
-    if (!immediate && reason() != ReasonRightClick)
+    if (!immediate && reason() != ReasonRightClick) {
         // We use a 0s single shot timer so that when left-clicking on a part,
         // we process the mouse event before rebuilding the GUI.
         // Otherwise, when e.g. dragging icons, the mouse pointer can already
         // be very far from where it was...
-        // TODO: use a QTimer member var, so that if two conflicting calls to
-        // setActivePart(part,immediate=false) happen, the 1st one gets cancelled.
-        QTimer::singleShot( 0, this, SLOT( emitActivePartChanged() ) );
+        m_activePartChangedTimer->start( 0, true );
         // This is not done with right-clicking so that the part is activated before the
         // popup appears (#75201)
-    else
+    } else {
         emitActivePartChanged();
+    }
 }
 
 void KonqViewManager::slotActivePartChanged ( KParts::Part *newPart )
@@ -1447,7 +1451,7 @@ void KonqViewManager::loadItem( KConfig &cfg, KonqFrameContainerBase *parent,
   if( name != "InitialView" )
     prefix = name + '_';
 
-  //kdDebug(1202) << "begin loadItem: " << name << endl;
+  //kdDebug(1202) << "KonqViewManager::loadItem: begin name " << name << " openURL " << openURL << endl;
 
   if( name.startsWith("View") || name == "empty" ) {
     //load view config
@@ -1462,7 +1466,7 @@ void KonqViewManager::loadItem( KConfig &cfg, KonqFrameContainerBase *parent,
         serviceType = cfg.readEntry( QString::fromLatin1( "ServiceType" ).prepend( prefix ), "inode/directory");
         serviceName = cfg.readEntry( QString::fromLatin1( "ServiceName" ).prepend( prefix ) );
     }
-    //kdDebug(1202) << "ServiceType: " << serviceType << " " << serviceName << endl;
+    //kdDebug(1202) << "KonqViewManager::loadItem: ServiceType " << serviceType << " " << serviceName << endl;
 
     KService::Ptr service;
     KTrader::OfferList partServiceOffers, appServiceOffers;
@@ -1476,7 +1480,7 @@ void KonqViewManager::loadItem( KConfig &cfg, KonqFrameContainerBase *parent,
 
     bool passiveMode = cfg.readBoolEntry( QString::fromLatin1( "PassiveMode" ).prepend( prefix ), false );
 
-    //kdDebug(1202) << "Creating View Stuff" << endl;
+    //kdDebug(1202) << "KonqViewManager::loadItem: Creating View Stuff" << endl;
     KonqView *childView = setupView( parent, viewFactory, service, partServiceOffers, appServiceOffers, serviceType, passiveMode, openAfterCurrentPage );
 
     if (!childView->isFollowActive()) childView->setLinkedView( cfg.readBoolEntry( QString::fromLatin1( "LinkedView" ).prepend( prefix ), false ) );
@@ -1535,6 +1539,7 @@ void KonqViewManager::loadItem( KConfig &cfg, KonqFrameContainerBase *parent,
     {
       KURL url;
 
+      //kdDebug(1202) << "KonqViewManager::loadItem: key " << key << endl;
       if ( cfg.hasKey( key ) )
       {
         QString u = cfg.readPathEntry( key );
@@ -1542,12 +1547,14 @@ void KonqViewManager::loadItem( KConfig &cfg, KonqFrameContainerBase *parent,
           u = QString::fromLatin1("about:blank");
         url = u;
       }
+      else if(key == "empty_URL")
+        url = QString::fromLatin1("about:blank");
       else
         url = defaultURL;
 
       if ( !url.isEmpty() )
       {
-        //kdDebug(1202) << "loadItem: calling openURL " << url.prettyURL() << endl;
+        //kdDebug(1202) << "KonqViewManager::loadItem: calling openURL " << url.prettyURL() << endl;
         //childView->openURL( url, url.prettyURL() );
         // We need view-follows-view (for the dirtree, for instance)
         KonqOpenURLRequest req;
@@ -1555,12 +1562,13 @@ void KonqViewManager::loadItem( KConfig &cfg, KonqFrameContainerBase *parent,
           req.typedURL = url.prettyURL();
         m_pMainWindow->openView( serviceType, url, childView, req );
       }
+      //else kdDebug(1202) << "KonqViewManager::loadItem: url is empty" << endl;
     }
     // Do this after opening the URL, so that it's actually possible to open it :)
     childView->setLockedLocation( cfg.readBoolEntry( QString::fromLatin1( "LockedLocation" ).prepend( prefix ), false ) );
   }
   else if( name.startsWith("Container") ) {
-    //kdDebug(1202) << "Item is Container" << endl;
+    //kdDebug(1202) << "KonqViewManager::loadItem Item is Container" << endl;
 
     //load container config
     QString ostr = cfg.readEntry( QString::fromLatin1( "Orientation" ).prepend( prefix ) );
@@ -1616,7 +1624,7 @@ void KonqViewManager::loadItem( KConfig &cfg, KonqFrameContainerBase *parent,
   }
   else if( name.startsWith("Tabs") )
   {
-    //kdDebug(1202) << "Item is a Tabs" << endl;
+    //kdDebug(1202) << "KonqViewManager::loadItem: Item is a Tabs" << endl;
 
     KonqFrameTabs *newContainer = new KonqFrameTabs( parent->widget(), parent, this );
     connect(newContainer,SIGNAL(ctrlTabPressed()),m_pMainWindow,SLOT(slotCtrlTabPressed()));
@@ -1635,7 +1643,7 @@ void KonqViewManager::loadItem( KConfig &cfg, KonqFrameContainerBase *parent,
           KonqView* activeChildView = dynamic_cast<KonqFrameBase*>(currentPage)->activeChildView();
           if (activeChildView != 0L) {
             activeChildView->setCaption( activeChildView->caption() );
-            activeChildView->setTabIcon( activeChildView->url().url() );
+            activeChildView->setTabIcon( activeChildView->url() );
           }
         }
     }
@@ -1648,7 +1656,7 @@ void KonqViewManager::loadItem( KConfig &cfg, KonqFrameContainerBase *parent,
   else
       kdWarning() << "Profile Loading Error: Unknown item " << name;
 
-  //kdDebug(1202) << "end loadItem: " << name << endl;
+  //kdDebug(1202) << "KonqViewManager::loadItem: end" << name << endl;
 }
 
 void KonqViewManager::setProfiles( KActionMenu *profiles )
