@@ -1,17 +1,26 @@
-/* ----------------------------------------------------------------------- */
-/*                                                                         */
-/* [konsole.h]                      Konsole                                   */
-/*                                                                            */
-/* -------------------------------------------------------------------------- */
-/*                                                                            */
-/* Copyright (c) 1997,1998 by Lars Doelle <lars.doelle@on-line.de>            */
-/*                                                                            */
-/* This file is part of Konsole, an X terminal.                               */
-/*                                                                            */
-/* The material contained in here more or less directly orginates from        */
-/* kvt, which is copyright (c) 1996 by Matthias Ettrich <ettrich@kde.org>     */
-/*                                                                            */
-/* -------------------------------------------------------------------------- */
+/*
+    This file is part of Konsole, an X terminal.
+    Copyright (C) 1996 by Matthias Ettrich <ettrich@kde.org>
+    Copyright (C) 1997,1998 by Lars Doelle <lars.doelle@on-line.de>
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA.
+*/
+/* The material contained in here more or less directly orginates from    */
+/* kvt, which is copyright (c) 1996 by Matthias Ettrich <ettrich@kde.org> */
+/*                                                                        */
 
 #ifndef KONSOLE_H
 #define KONSOLE_H
@@ -42,7 +51,7 @@
 #undef PACKAGE
 #undef VERSION
 #define PACKAGE "konsole"
-#define VERSION "1.6.2"
+#define VERSION "1.6.6"
 
 class KRootPixmap;
 class QLabel;
@@ -139,7 +148,7 @@ private slots:
   void activateSession();
   void activateSession(TESession*);
   void closeCurrentSession();
-  void confirmCloseCurrentSession();
+  void confirmCloseCurrentSession(TESession* _se=0);
   void doneSession(TESession*);
   void slotCouldNotClose();
   void toggleFullScreen();
@@ -163,7 +172,7 @@ private slots:
   void changeColLin(int columns, int lines);
   void notifySessionState(TESession* session,int state);
   void notifySize(int columns, int lines);
-  void updateTitle();
+  void updateTitle(TESession* _se=0);
   void prevSession();
   void nextSession();
   void activateMenu();
