@@ -80,7 +80,7 @@ KEyeCandyPage::KEyeCandyPage(QWidget *parent, const char *name ) : KEyeCandyPage
 	backgrounds_panel= new QCheckListItem(klv_features, i18n("Panel Background Picture"),
 				QCheckListItem::CheckBox);
 	// Level 3
-	icon_zooming_panel = new QCheckListItem(klv_features, i18n("Panel Icon Zooming"),
+	icon_zooming_panel = new QCheckListItem(klv_features, i18n("Panel Icon Popups"),
 				QCheckListItem::CheckBox);
 	icon_effect_gamma = new QCheckListItem(klv_features, i18n("Icon Highlighting"),
 				QCheckListItem::CheckBox);
@@ -184,7 +184,7 @@ void KEyeCandyPage::slotEyeCandySliderMoved(int value){
 	if( value >= 3){
 		// Level 3
 		icon_effect_gamma->setOn(true);
-		if(osStyle=="mac" && !(sys->isXfromXFreeInc() && sys->getXRelease()==40100000) )
+		if(!(sys->isXfromXFreeInc() && sys->getXRelease()==40100000) )
 			icon_zooming_panel->setOn(true);
 	}
 	if( value >= 4){
@@ -216,9 +216,6 @@ void KEyeCandyPage::slotEyeCandySliderMoved(int value){
 	if( value >= 8){
 		// Level 8
 		preview_text->setOn(true);
-		//icon-zoom (now for all selections)
-		if ( !(sys->isXfromXFreeInc() && sys->getXRelease()==40100000) )
-			icon_zooming_panel->setOn(true);
 		// icon-size (now for all selections)
 		for (int i = 0; i < QApplication::desktop()->numScreens(); i++) {
 			if(QApplication::desktop()->screenGeometry(i).width() >= 1024)
@@ -243,17 +240,17 @@ void KEyeCandyPage::enableDesktopWallpaper(bool enable, bool user){
 	kdesktopconf->setGroup("Desktop0");
 
 	if( st_UserWallpaper.WallpaperMode == "NoWallpaper")
-	deskbgimage="default_gears.jpg";
+	deskbgimage="KDE34.png";
 
 	if(enable && !user){
 		// if the user has a different mode than the default of NoMulti, we don't change anyting on that.
 		if( st_UserWallpaper.MultiWallpaperMode == "NoMulti" )
 			kdesktopconf->writeEntry("MultiWallpaperMode", "NoMulti");
 		// if the wallpaper is the new default one, set mode to scaled to leave user settings untouched
-		if( deskbgimage == "default_gears.jpg"){
+		if( deskbgimage == "KDE34.png"){
 			kdesktopconf->writeEntry("WallpaperMode", "Scaled");
 			//here we change the kdesktop font color to white as it fits better
-			// to the default_gears.jpg gray background
+			// to the KDE34png.png gray background
 			kdesktopconf->setGroup("FMSettings");
 			kdesktopconf->writeEntry("NormalTextColor", QColor("#FFFFFF") );
 			kdesktopconf->setGroup("Desktop0");
@@ -350,8 +347,8 @@ void KEyeCandyPage::enableDesktopWindowMovingContents(bool enable, bool restore)
 //---------------------------BACKGROUNDS--------------------------------------------------
 /** Here, the background tiles/wallpapers for Konqueror and Kicker are set to the default values. Enabled in Level 2. */
 void KEyeCandyPage::enableBackgroundsPanel(bool enable){
-	// TODO: come up with a good background
-	Q_UNUSED(enable);
+	kickerconf->setGroup("General");
+	kickerconf->writeEntry("UseBackgroundTheme", enable);
 }
 
 /** Here, the background tiles/wallpapers for Konqueror and Kicker are set to the default values. Enabled in Level 2. */
@@ -597,32 +594,13 @@ void KEyeCandyPage::enableSoundScheme(bool enable, bool user) {
 void KEyeCandyPage::enableAntialiasingFonts(bool enable, bool reset){
 	if (!reset){
 		if ( ( (sys->isXfromXFreeInc() && (sys->getXRelease() > 40000000)) || sys->isXfromXOrg() )
-		     && sys->getRenderSupport()
-		     && (*userFonts.at(1)).contains("elvetica")) {
+		     && sys->getRenderSupport()) {
 			QSettings().writeEntry("/qt/useXft", enable);
 			QSettings().writeEntry("/qt/enableXft", enable);
-			if (enable) {
-				KGlobal::config()->setGroup("General");
-				KGlobal::config()->writeEntry("fixed", sys->getFixedWidthFont().toString(), true, true);
-				KGlobal::config()->writeEntry("font", sys->getNormalFont().toString(), true, true);
-				KGlobal::config()->writeEntry("menuFont", sys->getNormalFont().toString(), true, true);
-				KGlobal::config()->writeEntry("taskbarFont", sys->getSmallFont().toString(), true, true);
-				KGlobal::config()->writeEntry("toolBarFont", sys->getNormalFont().toString(), true, true);
-				KGlobal::config()->setGroup("WM");
-				KGlobal::config()->writeEntry("activeFont", sys->getBoldFont().toString(), true, true);
-			}
 		}
 	} else {
 		QSettings().writeEntry("/qt/useXft", b_useXft);
 		QSettings().writeEntry("/qt/enableXft", b_enableXft);
-		KGlobal::config()->setGroup("General");
-		KGlobal::config()->writeEntry("fixed", *userFonts.at(0), true, true);
-		KGlobal::config()->writeEntry("font", *userFonts.at(1), true, true);
-		KGlobal::config()->writeEntry("menuFont", *userFonts.at(2), true, true);
-		KGlobal::config()->writeEntry("taskbarFont", *userFonts.at(3), true, true);
-		KGlobal::config()->writeEntry("toolBarFont", *userFonts.at(4), true, true);
-		KGlobal::config()->setGroup("WM");
-		KGlobal::config()->writeEntry("activeFont", *userFonts.at(5), true, true);
 	}
 }
 //----------------------------OTHER STUFF-------------------------------------------------
@@ -711,7 +689,7 @@ void KEyeCandyPage::getUserDefaults(){
 	st_UserWallpaper.MultiWallpaperMode = kdesktopconf->readEntry("MultiWallpaperMode", "NoMulti");
 	st_UserWallpaper.WallpaperMode = kdesktopconf->readEntry("WallpaperMode", "Scaled");
 	st_UserWallpaper.Wallpaper = kdesktopconf->readPathEntry("Wallpaper", "NoWallpaper");
-	deskbgimage = kdesktopconf->readPathEntry("Wallpaper", "default_gears.jpg");
+	deskbgimage = kdesktopconf->readPathEntry("Wallpaper", "KDE34.png");
 	// Wallpaper-User-Defaults (END)
 
 	KGlobal::config()->setGroup("KDE");
@@ -719,14 +697,6 @@ void KEyeCandyPage::getUserDefaults(){
 	// Font-User-Defaults
 	b_useXft=QSettings().readBoolEntry("/qt/useXft");
 	b_enableXft=QSettings().readBoolEntry("/qt/enableXft");
-	KGlobal::config()->setGroup("General");
-	userFonts.append(KGlobal::config()->readEntry("fixed", "courier,10,-1,5,50,0,0,0,0,0"));
-	userFonts.append(KGlobal::config()->readEntry("font", "helvetica,12,-1,5,50,0,0,0,0,0"));
-	userFonts.append(KGlobal::config()->readEntry("menuFont", "helvetica,12,-1,5,50,0,0,0,0,0"));
-	userFonts.append(KGlobal::config()->readEntry("taskbarFont", "helvetica,11,-1,5,50,0,0,0,0,0"));
-	userFonts.append(KGlobal::config()->readEntry("toolBarFont", "helvetica,12,-1,5,50,0,0,0,0,0"));
-	KGlobal::config()->setGroup("WM");
-	userFonts.append(KGlobal::config()->readEntry("activeFont", "helvetica,12,-1,5,75,0,0,0,0,0"));
 
 	KGlobal::config()->setGroup("PanelIcons");
 	st_UserGamma.EffectPanel=KGlobal::config()->readEntry("ActiveEffect", "none");
@@ -869,7 +839,6 @@ void KEyeCandyPage::saveCheckState(bool currSettings){ // currSettings= true ->
 		enableIconEffectGamma(false, true);
 		enableIconMngAnimation(b_iconMngAnim);
 		enableSoundScheme(false, true);
-		// misses functionality at all
 		enableBackgroundsPanel(false);
 	}
 }
