@@ -67,12 +67,14 @@ class Client : public QObject, public KDecorationDefines
         const Group* group() const;
         Group* group();
         void checkGroup( Group* gr = NULL, bool force = false );
+        void changeClientLeaderGroup( Group* gr );
     // prefer isXXX() instead
         NET::WindowType windowType( bool direct = false, int supported_types = SUPPORTED_WINDOW_TYPES_MASK ) const;
         const WindowRules* rules() const;
         void removeRule( Rules* r );
         void setupWindowRules( bool ignore_temporary );
         void applyWindowRules();
+        void updateWindowRules();
 
         QRect geometry() const;
         QSize size() const;
@@ -301,6 +303,7 @@ class Client : public QObject, public KDecorationDefines
         void setBMP(bool b);
         bool touches(const Client* c);
         void setShapable(bool b);
+        bool hasStrut() const;
 
     private slots:
         void autoRaise();
@@ -335,7 +338,6 @@ class Client : public QObject, public KDecorationDefines
         void clientMessageEvent( XClientMessageEvent* e );
         void enterNotifyEvent( XCrossingEvent* e );
         void leaveNotifyEvent( XCrossingEvent* e );
-        void visibilityNotifyEvent( XVisibilityEvent* e );
         void focusInEvent( XFocusInEvent* e );
         void focusOutEvent( XFocusOutEvent* e );
 
@@ -361,7 +363,7 @@ class Client : public QObject, public KDecorationDefines
         QSize sizeForClientSize( const QSize&, Sizemode mode = SizemodeAny, bool noframe = false ) const;
         void changeMaximize( bool horizontal, bool vertical, bool adjust );
         void checkMaximizeGeometry();
-        bool checkFullScreenHack( const QRect& geom ) const;
+        int checkFullScreenHack( const QRect& geom ) const; // 0 - none, 1 - one xinerama screen, 2 - full area
         void updateFullScreenHack( const QRect& geom );
         void getWmNormalHints();
         void getMotifHints();
@@ -373,7 +375,6 @@ class Client : public QObject, public KDecorationDefines
         QString readName() const;
         void setCaption( const QString& s, bool force = false );
         bool hasTransientInternal( const Client* c, bool indirect, ConstClientList& set ) const;
-        void updateWindowRules();
         void finishWindowRules();
         void setShortcutInternal( const KShortcut& cut );
 
@@ -382,7 +383,6 @@ class Client : public QObject, public KDecorationDefines
         static int computeWorkareaDiff( int left, int right, int a_left, int a_right );
         void configureRequest( int value_mask, int rx, int ry, int rw, int rh, int gravity, bool from_tool );
         NETExtendedStrut strut() const;
-        bool hasStrut() const;
         int checkShadeGeometry( int w, int h );
         void postponeGeometryUpdates( bool postpone );
 
@@ -485,7 +485,6 @@ class Client : public QObject, public KDecorationDefines
         uint modal : 1; // NET::Modal
         uint noborder : 1;
         uint user_noborder : 1;
-        uint not_obscured : 1;
         uint urgency : 1; // XWMHints, UrgencyHint
         uint ignore_focus_stealing : 1; // don't apply focus stealing prevention to this client
         uint demands_attention : 1;
@@ -546,6 +545,8 @@ class Client : public QObject, public KDecorationDefines
         //int shadeOriginalHeight;
         bool isBMP_;
         QTimer* demandAttentionKNotifyTimer;
+
+        friend bool performTransiencyCheck();
     };
 
 // helper for Client::postponeGeometryUpdates() being called in pairs (true/false)
