@@ -43,6 +43,12 @@ namespace KWinInternal
  */
 void Workspace::desktopResized()
     {
+    QRect geom = QApplication::desktop()->geometry();
+    NETSize desktop_geometry;
+    desktop_geometry.width = geom.width();
+    desktop_geometry.height = geom.height();
+    rootInfo->setDesktopGeometry( -1, desktop_geometry );
+
     updateClientArea();
     checkElectricBorders( true );
     }
@@ -210,7 +216,7 @@ QRect Workspace::clientArea( clientAreaOption opt, const QPoint& p, int desktop
     if( desktop == NETWinInfo::OnAllDesktops || desktop == 0 )
         desktop = currentDesktop();
     QDesktopWidget *desktopwidget = KApplication::desktop();
-    int screen = desktopwidget->screenNumber( p );
+    int screen = desktopwidget->isVirtualDesktop() ? desktopwidget->screenNumber( p ) : desktopwidget->primaryScreen();
     if( screen < 0 )
         screen = desktopwidget->primaryScreen();
     QRect sarea = screenarea // may be NULL during KWin initialization
