@@ -117,7 +117,11 @@ void Workspace::updateStackingOrder( bool propagate_new_clients )
         }
 #endif
     if( changed || propagate_new_clients )
+        {
         propagateClients( propagate_new_clients );
+        if( active_client )
+            active_client->updateMouseGrab();
+        }
     }
 
 /*!
@@ -190,7 +194,7 @@ void Workspace::propagateClients( bool propagate_new_clients )
   doesn't accept focus it's excluded.
  */
 // TODO misleading name for this method
-Client* Workspace::topClientOnDesktop( int desktop, bool unconstrained ) const
+Client* Workspace::topClientOnDesktop( int desktop, bool unconstrained, bool only_normal ) const
     {
 // TODO    Q_ASSERT( block_stacking_updates == 0 );
     ClientList::ConstIterator begin, end;
@@ -208,9 +212,13 @@ Client* Workspace::topClientOnDesktop( int desktop, bool unconstrained ) const
         it != end;
         --it )
         {
-        if ( (*it)->isOnDesktop( desktop ) && !(*it)->isSpecialWindow()
-            && (*it)->isShown( false ) && (*it)->wantsTabFocus())
-            return *it;
+        if( (*it)->isOnDesktop( desktop ) && (*it)->isShown( false ))
+            {
+            if( !only_normal )
+                return *it;
+            if( (*it)->wantsTabFocus() && !(*it)->isSpecialWindow())
+                return *it;
+            }
         }
     return 0;
     }
@@ -404,20 +412,27 @@ void Workspace::restackClientUnderActive( Client* c )
         return;
         }
 
-    // put in the stacking order below _all_ windows belonging to the active application
     assert( unconstrained_stacking_order.contains( active_client ));
-    for( ClientList::Iterator it = unconstrained_stacking_order.begin();
-         it != unconstrained_stacking_order.end();
-         ++it )
-        { // TODO ignore topmenus?
-        if( Client::belongToSameApplication( active_client, *it ))
-            {
-            if( *it != c )
+    if( Client::belongToSameApplication( active_client, c ))
+        { // put it below the active window if it's the same app
+        unconstrained_stacking_order.remove( c );
+        unconstrained_stacking_order.insert( unconstrained_stacking_order.find( active_client ), c );
+        }
+    else
+        { // put in the stacking order below _all_ windows belonging to the active application
+        for( ClientList::Iterator it = unconstrained_stacking_order.begin();
+             it != unconstrained_stacking_order.end();
+             ++it )
+            { // TODO ignore topmenus?
+            if( Client::belongToSameApplication( active_client, *it ))
                 {
-                unconstrained_stacking_order.remove( c );
-                unconstrained_stacking_order.insert( it, c );
+                if( *it != c )
+                    {
+                    unconstrained_stacking_order.remove( c );
+                    unconstrained_stacking_order.insert( it, c );
+                    }
+                break;
                 }
-            break;
             }
         }
     assert( unconstrained_stacking_order.contains( c ));
@@ -427,16 +442,23 @@ void Workspace::restackClientUnderActive( Client* c )
         { // do for every virtual desktop to handle the case of onalldesktop windows
         if( c->wantsTabFocus() && c->isOnDesktop( desktop ) && focus_chain[ desktop ].contains( active_client ))
             {
-            // also put in focus_chain[currentDesktop()] after all windows belonging to the active applicationa
-            focus_chain[ desktop ].remove( c );
-            for( ClientList::Iterator it = focus_chain[ desktop ].fromLast();
-                 it != focus_chain[ desktop ].end();
-                 --it )
-      	        {
-                if( Client::belongToSameApplication( active_client, *it ))
-                    {
-                    focus_chain[ desktop ].insert( it, c );
-                    break;
+            if( Client::belongToSameApplication( active_client, c ))
+                { // put it after the active window if it's the same app
+                focus_chain[ desktop ].remove( c );
+                focus_chain[ desktop ].insert( focus_chain[ desktop ].find( active_client ), c );
+                }
+            else
+                { // put it in focus_chain[currentDesktop()] after all windows belonging to the active applicationa
+                focus_chain[ desktop ].remove( c );
+                for( ClientList::Iterator it = focus_chain[ desktop ].fromLast();
+                     it != focus_chain[ desktop ].end();
+                     --it )
+          	        {
+                    if( Client::belongToSameApplication( active_client, *it ))
+                        {
+                        focus_chain[ desktop ].insert( it, c );
+                        break;
+                        }
                     }
                 }
             }
@@ -444,15 +466,23 @@ void Workspace::restackClientUnderActive( Client* c )
     // the same for global_focus_chain
     if( c->wantsTabFocus() && global_focus_chain.contains( active_client ))
         {
-        global_focus_chain.remove( c );
-        for( ClientList::Iterator it = global_focus_chain.fromLast();
-             it != global_focus_chain.end();
-             --it )
+        if( Client::belongToSameApplication( active_client, c ))
             {
-            if( Client::belongToSameApplication( active_client, *it ))
+            global_focus_chain.remove( c );
+            global_focus_chain.insert( global_focus_chain.find( active_client ), c );
+            }
+        else
+            {
+            global_focus_chain.remove( c );
+            for( ClientList::Iterator it = global_focus_chain.fromLast();
+                 it != global_focus_chain.end();
+                 --it )
                 {
-                global_focus_chain.insert( it, c );
-                break;
+                if( Client::belongToSameApplication( active_client, *it ))
+                    {
+                    global_focus_chain.insert( it, c );
+                    break;
+                    }
                 }
             }
       	}
