@@ -23,6 +23,7 @@ License. See the file "COPYING" for the exact licensing terms.
 
 #include "notifications.h"
 #include "rules.h"
+#include "group.h"
 
 extern Time qt_x_time;
 
@@ -104,6 +105,7 @@ bool Client::manage( Window w, bool isMapped )
     setCaption( cap_normal, true );
 
     detectNoBorder();
+    detectShapable();
     fetchIconicName();
     getWMHints(); // needs to be done before readTransient() because of reading the group
     modal = ( info->state() & NET::Modal ) != 0; // needs to be valid before handling groups
@@ -164,7 +166,7 @@ bool Client::manage( Window w, bool isMapped )
                  it != mainclients.end();
                  ++it )
                 {
-                if( (*it)->isSpecialWindow() && !(*it)->isOverride())
+                if( (*it)->isSpecialWindow())
                     continue; // don't consider toolbars etc when placing
                 maincl = *it;
                 if( (*it)->isOnCurrentDesktop())
@@ -204,10 +206,17 @@ bool Client::manage( Window w, bool isMapped )
     else
         area = workspace()->clientArea( PlacementArea, geom.center(), desktop());
 
-    if( checkFullScreenHack( geom ))
+    if( int type = checkFullScreenHack( geom ))
         {
         fullscreen_mode = FullScreenHack;
-        geom = workspace()->clientArea( FullScreenArea, geom.center(), desktop());
+        if( rules()->checkStrictGeometry( false ))
+            {
+            geom = type == 2 // 1 - it's xinerama-aware fullscreen hack, 2 - it's full area
+                ? workspace()->clientArea( FullArea, geom.center(), desktop())
+                : workspace()->clientArea( ScreenArea, geom.center(), desktop());
+            }
+        else
+            geom = workspace()->clientArea( FullScreenArea, geom.center(), desktop());
         placementDone = true;
         }
 
@@ -250,7 +259,7 @@ bool Client::manage( Window w, bool isMapped )
         ; // force using placement policy
     else
         usePosition = true;
-    if( !rules()->checkIgnorePosition( !usePosition ))
+    if( !rules()->checkIgnoreGeometry( !usePosition ))
         {
         bool ignorePPosition = ( options->ignorePositionClasses.contains(QString::fromLatin1(resourceClass())));
 
@@ -307,12 +316,8 @@ bool Client::manage( Window w, bool isMapped )
         keepInArea( area, partial_keep_in_area );
 
     XShapeSelectInput( qt_xdisplay(), window(), ShapeNotifyMask );
-    if ( (is_shape = Shape::hasShape( window())) ) 
-        {
-        updateShape();
-        }
-//    else
-//	setShapable(FALSE);
+    is_shape = Shape::hasShape( window());
+    updateShape();
 	
     //CT extra check for stupid jdk 1.3.1. But should make sense in general
     // if client has initial state set to Iconic and is transient with a parent
@@ -421,12 +426,14 @@ bool Client::manage( Window w, bool isMapped )
     // - keep it?
     XLowerWindow( qt_xdisplay(), frameId());
 
+    // set initial user time directly
     user_time = readUserTimeMapTimestamp( asn_valid ? &asn_id : NULL, asn_valid ? &asn_data : NULL, session );
+    group()->updateUserTime( user_time ); // and do what Client::updateUserTime() does
 
     if( isTopMenu()) // they're shown in Workspace::addClient() if their mainwindow
         hideClient( true ); // is the active one
 
-    if ( isShown( true ) && !doNotShow )
+    if( isShown( true ) && !doNotShow )
         {
         if( isDialog())
             Notify::raise( Notify::TransNew );
@@ -435,7 +442,9 @@ bool Client::manage( Window w, bool isMapped )
 
         bool allow;
         if( session )
-            allow = session->active && !workspace()->wasUserInteraction();
+            allow = session->active
+                && ( !workspace()->wasUserInteraction()
+                    || workspace()->activeClient() == NULL || workspace()->activeClient()->isDesktop());
         else
             allow = workspace()->allowClientActivation( this, userTime(), false );
 
@@ -444,46 +453,43 @@ bool Client::manage( Window w, bool isMapped )
         if( !isOnCurrentDesktop() && !isMapped && !session && ( allow || workspace()->sessionSaving()))
             workspace()->setCurrentDesktop( desktop());
 
-        if( isOnCurrentDesktop())
-            {
-            setMappingState( NormalState );
-
-            if( isMapped )
+        bool belongs_to_desktop = false;
+        for( ClientList::ConstIterator it = group()->members().begin();
+             it != group()->members().end();
+             ++it )
+            if( (*it)->isDesktop())
                 {
-                workspace()->raiseClient( this );
-                rawShow();
+                belongs_to_desktop = true;
+                break;
+                }
+        if( !belongs_to_desktop && workspace()->showingDesktop())
+            workspace()->resetShowingDesktop( options->showDesktopIsMinimizeAll );
+
+        if( isOnCurrentDesktop() && !isMapped && !allow )
+            workspace()->restackClientUnderActive( this );
+        else
+            workspace()->raiseClient( this );
+
+        updateVisibility();
+
+        if( !isMapped )
+            {
+            if( allow && isOnCurrentDesktop())
+                {
+                if( !isSpecialWindow())
+                    if ( options->focusPolicyIsReasonable() && wantsTabFocus() )
+                        workspace()->requestFocus( this );
                 }
             else
                 {
-                if( allow )
-                    {
-                    workspace()->raiseClient( this );
-                    rawShow();
-                    if( !isSpecialWindow() || isOverride())
-                        if ( options->focusPolicyIsReasonable() && wantsTabFocus() )
-                            workspace()->requestFocus( this );
-                    }
-                else
-                    {
-                    workspace()->restackClientUnderActive( this );
-                    rawShow();
-                    if( !session && ( !isSpecialWindow() || isOverride()))
+                if( !session && !isSpecialWindow())
                         demandAttention();
-                    }
                 }
             }
-        else
-            {
-            virtualDesktopChange();
-            workspace()->raiseClient( this );
-            if( !session && !isMapped )
-                demandAttention();
-            }
         }
-    else if( !doNotShow ) // !isShown()
+    else if( !doNotShow ) // if( !isShown( true ) && !doNotShow )
         {
-        rawHide();
-        setMappingState( IconicState );
+        updateVisibility();
         }
     else // doNotShow
         { // SELI HACK !!!
@@ -508,6 +514,8 @@ bool Client::manage( Window w, bool isMapped )
     ungrabXServer();
     
     client_rules.discardTemporary();
+    applyWindowRules(); // just in case
+    workspace()->discardUsedWindowRules( this, false ); // remove ApplyNow rules
     updateWindowRules(); // was blocked while !isManaged()
 
 // TODO there's a small problem here - isManaged() depends on the mapping state,
@@ -557,8 +565,7 @@ void Client::embedClient( Window w, const XWindowAttributes &attr )
             FocusChangeMask |
             ExposureMask |
             PropertyChangeMask |
-            StructureNotifyMask | SubstructureRedirectMask |
-            VisibilityChangeMask );
+            StructureNotifyMask | SubstructureRedirectMask );
     XSelectInput( qt_xdisplay(), wrapper, ClientWinMask | SubstructureNotifyMask );
     XSelectInput( qt_xdisplay(), client,
                   FocusChangeMask |
