@@ -1,4 +1,4 @@
-/********x*********************************************************
+/*****************************************************************
  KWin - the KDE window manager
  This file is part of the KDE project.
 
@@ -44,7 +44,7 @@ namespace KWinInternal
 extern QPixmap* kwin_get_menu_pix_hack();
 
 TabBox::TabBox( Workspace *ws, const char *name )
-    : QFrame( 0, name, Qt::WNoAutoErase ), client(0), wspace(ws)
+    : QFrame( 0, name, Qt::WNoAutoErase ), current_client( NULL ), wspace(ws)
     {
     setFrameStyle(QFrame::StyledPanel | QFrame::Plain);
     setLineWidth(2);
@@ -57,10 +57,25 @@ TabBox::TabBox( Workspace *ws, const char *name )
     reconfigure();
     reset();
     connect(&delayedShowTimer, SIGNAL(timeout()), this, SLOT(show()));
+    
+    XSetWindowAttributes attr;
+    attr.override_redirect = 1;
+    outline_left = XCreateWindow( qt_xdisplay(), qt_xrootwin(), 0, 0, 1, 1, 0,
+        CopyFromParent, CopyFromParent, CopyFromParent, CWOverrideRedirect, &attr );
+    outline_right = XCreateWindow( qt_xdisplay(), qt_xrootwin(), 0, 0, 1, 1, 0,
+        CopyFromParent, CopyFromParent, CopyFromParent, CWOverrideRedirect, &attr );
+    outline_top = XCreateWindow( qt_xdisplay(), qt_xrootwin(), 0, 0, 1, 1, 0,
+        CopyFromParent, CopyFromParent, CopyFromParent, CWOverrideRedirect, &attr );
+    outline_bottom = XCreateWindow( qt_xdisplay(), qt_xrootwin(), 0, 0, 1, 1, 0,
+        CopyFromParent, CopyFromParent, CopyFromParent, CWOverrideRedirect, &attr );
     }
 
 TabBox::~TabBox()
     {
+    XDestroyWindow( qt_xdisplay(), outline_left );
+    XDestroyWindow( qt_xdisplay(), outline_right );
+    XDestroyWindow( qt_xdisplay(), outline_top );
+    XDestroyWindow( qt_xdisplay(), outline_bottom );
     }
 
 
@@ -111,7 +126,9 @@ void TabBox::createClientList(ClientList &list, int desktop /*-1 = all*/, Client
                 else if( !list.contains( modal ))
                     list += modal;
                 else
-                    ; // nothing
+                    {
+                    // nothing
+                    }
                 }
             }
 
@@ -147,10 +164,10 @@ void TabBox::reset()
 
     if ( mode() == WindowsMode )
         {
-        client = workspace()->activeClient();
+        setCurrentClient( workspace()->activeClient());
 
         // get all clients to show
-        createClientList(clients, options_traverse_all ? -1 : workspace()->currentDesktop(), client, true);
+        createClientList(clients, options_traverse_all ? -1 : workspace()->currentDesktop(), current_client, true);
 
         // calculate maximum caption width
         cw = fontMetrics().width(no_tasks)+20;
@@ -227,7 +244,8 @@ void TabBox::nextPrev( bool next)
     {
     if ( mode() == WindowsMode )
         {
-        Client* firstClient = 0;
+        Client* firstClient = NULL;
+        Client* client = current_client;
         do
             {
             if ( next )
@@ -247,6 +265,7 @@ void TabBox::nextPrev( bool next)
                 break;
                 }
             } while ( client && !clients.contains( client ));
+        setCurrentClient( client );
         }
     else if( mode() == DesktopMode )
         {
@@ -284,9 +303,18 @@ Client* TabBox::currentClient()
     {
     if ( mode() != WindowsMode )
         return 0;
-    if (!workspace()->hasClient( client ))
+    if (!workspace()->hasClient( current_client ))
         return 0;
-    return client;
+    return current_client;
+    }
+
+void TabBox::setCurrentClient( Client* c )
+    {
+    if( current_client != c )
+        {
+        current_client = c;
+        updateOutline();
+        }
     }
 
 /*!
@@ -308,6 +336,11 @@ int TabBox::currentDesktop()
  */
 void TabBox::showEvent( QShowEvent* )
     {
+    updateOutline();
+    XRaiseWindow( qt_xdisplay(), outline_left );
+    XRaiseWindow( qt_xdisplay(), outline_right );
+    XRaiseWindow( qt_xdisplay(), outline_top );
+    XRaiseWindow( qt_xdisplay(), outline_bottom );
     raise();
     }
 
@@ -317,6 +350,10 @@ void TabBox::showEvent( QShowEvent* )
  */
 void TabBox::hideEvent( QHideEvent* )
     {
+    XUnmapWindow( qt_xdisplay(), outline_left );
+    XUnmapWindow( qt_xdisplay(), outline_right );
+    XUnmapWindow( qt_xdisplay(), outline_top );
+    XUnmapWindow( qt_xdisplay(), outline_bottom );
     }
 
 /*!
@@ -355,7 +392,7 @@ void TabBox::drawContents( QPainter * )
               if ( workspace()->hasClient( *it ) )  // safety
                   {
                   // draw highlight background
-                  if ( (*it) == currentClient() )
+                  if ( (*it) == current_client )
                     p.fillRect(x, y, r.width(), lineHeight, colorGroup().highlight());
 
                   // draw icon
@@ -392,7 +429,7 @@ void TabBox::drawContents( QPainter * )
                   s = KStringHandler::cPixelSqueeze(s, fontMetrics(), r.width() - 5 - iconWidth - 8);
 
                   // draw text
-                  if ( (*it) == currentClient() )
+                  if ( (*it) == current_client )
                     p.setPen(colorGroup().highlightedText());
                   else if( (*it)->isMinimized())
                     {
@@ -508,6 +545,91 @@ void TabBox::drawContents( QPainter * )
     bitBlt(this, r.x(), r.y(), &pix);
     }
 
+void TabBox::updateOutline()
+    {
+    Client* c = currentClient();
+    if( !options->tabboxOutline || c == NULL || this->isHidden() || !c->isShown( true ) || !c->isOnCurrentDesktop())
+        {
+        XUnmapWindow( qt_xdisplay(), outline_left );
+        XUnmapWindow( qt_xdisplay(), outline_right );
+        XUnmapWindow( qt_xdisplay(), outline_top );
+        XUnmapWindow( qt_xdisplay(), outline_bottom );
+        return;
+        }
+    // left/right parts are between top/bottom, they don't reach as far as the corners
+    XMoveResizeWindow( qt_xdisplay(), outline_left, c->x(), c->y() + 5, 5, c->height() - 10 );
+    XMoveResizeWindow( qt_xdisplay(), outline_right, c->x() + c->width() - 5, c->y() + 5, 5, c->height() - 10 );
+    XMoveResizeWindow( qt_xdisplay(), outline_top, c->x(), c->y(), c->width(), 5 );
+    XMoveResizeWindow( qt_xdisplay(), outline_bottom, c->x(), c->y() + c->height() - 5, c->width(), 5 );
+    {
+    QPixmap pix( 5, c->height() - 10 );
+    QPainter p( &pix );
+    p.setPen( white );
+    p.drawLine( 0, 0, 0, pix.height() - 1 );
+    p.drawLine( 4, 0, 4, pix.height() - 1 );
+    p.setPen( gray );
+    p.drawLine( 1, 0, 1, pix.height() - 1 );
+    p.drawLine( 3, 0, 3, pix.height() - 1 );
+    p.setPen( black );
+    p.drawLine( 2, 0, 2, pix.height() - 1 );
+    p.end();
+    XSetWindowBackgroundPixmap( qt_xdisplay(), outline_left, pix.handle());
+    XSetWindowBackgroundPixmap( qt_xdisplay(), outline_right, pix.handle());
+    }
+    {
+    QPixmap pix( c->width(), 5 );
+    QPainter p( &pix );
+    p.setPen( white );
+    p.drawLine( 0, 0, pix.width() - 1 - 0, 0 );
+    p.drawLine( 4, 4, pix.width() - 1 - 4, 4 );
+    p.drawLine( 0, 0, 0, 4 );
+    p.drawLine( pix.width() - 1 - 0, 0, pix.width() - 1 - 0, 4 );
+    p.setPen( gray );
+    p.drawLine( 1, 1, pix.width() - 1 - 1, 1 );
+    p.drawLine( 3, 3, pix.width() - 1 - 3, 3 );
+    p.drawLine( 1, 1, 1, 4 );
+    p.drawLine( 3, 3, 3, 4 );
+    p.drawLine( pix.width() - 1 - 1, 1, pix.width() - 1 - 1, 4 );
+    p.drawLine( pix.width() - 1 - 3, 3, pix.width() - 1 - 3, 4 );
+    p.setPen( black );
+    p.drawLine( 2, 2, pix.width() - 1 - 2, 2 );
+    p.drawLine( 2, 2, 2, 4 );
+    p.drawLine( pix.width() - 1 - 2, 2, pix.width() - 1 - 2, 4 );
+    p.end();
+    XSetWindowBackgroundPixmap( qt_xdisplay(), outline_top, pix.handle());
+    }
+    {
+    QPixmap pix( c->width(), 5 );
+    QPainter p( &pix );
+    p.setPen( white );
+    p.drawLine( 4, 0, pix.width() - 1 - 4, 0 );
+    p.drawLine( 0, 4, pix.width() - 1 - 0, 4 );
+    p.drawLine( 0, 4, 0, 0 );
+    p.drawLine( pix.width() - 1 - 0, 4, pix.width() - 1 - 0, 0 );
+    p.setPen( gray );
+    p.drawLine( 3, 1, pix.width() - 1 - 3, 1 );
+    p.drawLine( 1, 3, pix.width() - 1 - 1, 3 );
+    p.drawLine( 3, 1, 3, 0 );
+    p.drawLine( 1, 3, 1, 0 );
+    p.drawLine( pix.width() - 1 - 3, 1, pix.width() - 1 - 3, 0 );
+    p.drawLine( pix.width() - 1 - 1, 3, pix.width() - 1 - 1, 0 );
+    p.setPen( black );
+    p.drawLine( 2, 2, pix.width() - 1 - 2, 2 );
+    p.drawLine( 2, 0, 2, 2 );
+    p.drawLine( pix.width() - 1 - 2, 0, pix.width() - 1 - 2, 2 );
+    p.end();
+    XSetWindowBackgroundPixmap( qt_xdisplay(), outline_bottom, pix.handle());
+    }
+    XClearWindow( qt_xdisplay(), outline_left );
+    XClearWindow( qt_xdisplay(), outline_right );
+    XClearWindow( qt_xdisplay(), outline_top );
+    XClearWindow( qt_xdisplay(), outline_bottom );
+    XMapWindow( qt_xdisplay(), outline_left );
+    XMapWindow( qt_xdisplay(), outline_right );
+    XMapWindow( qt_xdisplay(), outline_top );
+    XMapWindow( qt_xdisplay(), outline_bottom );
+    }
+
 void TabBox::hide()
     {
     delayedShowTimer.stop();
@@ -523,7 +645,7 @@ void TabBox::reconfigure()
     {
     KConfig * c(KGlobal::config());
     c->setGroup("TabBox");
-    options_traverse_all = c->readNumEntry("TraverseAll", false );
+    options_traverse_all = c->readBoolEntry("TraverseAll", false );
     }
 
 /*!
@@ -548,7 +670,7 @@ void TabBox::delayedShow()
     {
     KConfig * c(KGlobal::config());
     c->setGroup("TabBox");
-    bool delay = c->readNumEntry("ShowDelay", true);
+    bool delay = c->readBoolEntry("ShowDelay", true);
 
     if (!delay)
         {
@@ -584,7 +706,7 @@ void TabBox::handleMouseEvent( XEvent* e )
             {
             if( workspace()->hasClient( *it ) && (num == 0) ) // safety
                 {
-                client = *it;
+                setCurrentClient( *it );
                 break;
                 }
             num--;
@@ -622,19 +744,17 @@ void TabBox::handleMouseEvent( XEvent* e )
  */
 
 static
-bool areKeySymXsDepressed( bool bAll, int nKeySyms, ... )
+bool areKeySymXsDepressed( bool bAll, const uint keySyms[], int nKeySyms )
     {
-    va_list args;
     char keymap[32];
 
     kdDebug(125) << "areKeySymXsDepressed: " << (bAll ? "all of " : "any of ") << nKeySyms << endl;
 
-    va_start( args, nKeySyms );
     XQueryKeymap( qt_xdisplay(), keymap );
 
     for( int iKeySym = 0; iKeySym < nKeySyms; iKeySym++ )
         {
-        uint keySymX = va_arg( args, uint );
+        uint keySymX = keySyms[ iKeySym ];
         uchar keyCodeX = XKeysymToKeycode( qt_xdisplay(), keySymX );
         int i = keyCodeX / 8;
         char mask = 1 << (keyCodeX - (i * 8));
@@ -691,18 +811,16 @@ static bool areModKeysDepressed( const KKeySequence& seq )
         }
     if( mod & KKey::WIN )
         {
-        // HACK: it would take a lot of code to determine whether the Win key
-        //  is associated with Super or Meta, so check for both
+        // It would take some code to determine whether the Win key
+        // is associated with Super or Meta, so check for both.
+        // See bug #140023 for details.
         rgKeySyms[nKeySyms++] = XK_Super_L;
         rgKeySyms[nKeySyms++] = XK_Super_R;
         rgKeySyms[nKeySyms++] = XK_Meta_L;
         rgKeySyms[nKeySyms++] = XK_Meta_R;
         }
 
-    // Is there a better way to push all 8 integer onto the stack?
-    return areKeySymXsDepressed( false, nKeySyms,
-        rgKeySyms[0], rgKeySyms[1], rgKeySyms[2], rgKeySyms[3],
-        rgKeySyms[4], rgKeySyms[5], rgKeySyms[6], rgKeySyms[7] );
+    return areKeySymXsDepressed( false, rgKeySyms, nKeySyms );
     }
 
 static bool areModKeysDepressed( const KShortcut& cut )
@@ -723,7 +841,7 @@ void Workspace::slotWalkThroughWindows()
         return;
     if ( tab_grab || control_grab )
         return;
-    if ( options->altTabStyle == Options::CDE  || !options->focusPolicyIsReasonable() )
+    if ( options->altTabStyle == Options::CDE || !options->focusPolicyIsReasonable())
         {
         //XUngrabKeyboard(qt_xdisplay(), qt_x_time); // need that because of accelerator raw mode
         // CDE style raise / lower
@@ -739,9 +857,7 @@ void Workspace::slotWalkThroughWindows()
         else
             // if the shortcut has no modifiers, don't show the tabbox,
             // don't grab, but simply go to the next window
-            // use the CDE style, because with KDE style it would cycle
-            // between the active and previously active window
-            CDEWalkThroughWindows( true );
+            KDEOneStepThroughWindows( true );
         }
     }
 
@@ -751,7 +867,7 @@ void Workspace::slotWalkBackThroughWindows()
         return;
     if( tab_grab || control_grab )
         return;
-    if ( options->altTabStyle == Options::CDE  || !options->focusPolicyIsReasonable() )
+    if ( options->altTabStyle == Options::CDE || !options->focusPolicyIsReasonable())
         {
         // CDE style raise / lower
         CDEWalkThroughWindows( false );
@@ -765,7 +881,7 @@ void Workspace::slotWalkBackThroughWindows()
             }
         else
             {
-            CDEWalkThroughWindows( false );
+            KDEOneStepThroughWindows( false );
             }
         }
     }
@@ -843,8 +959,9 @@ bool Workspace::startKDEWalkThroughWindows()
     if( !establishTabBoxGrab())
         return false;
     tab_grab        = TRUE;
-    keys->setEnabled( false );
-    client_keys->setEnabled( false );
+    keys->suspend( true );
+    disable_shortcuts_keys->suspend( true );
+    client_keys->suspend( true );
     tab_box->setMode( TabBox::WindowsMode );
     tab_box->reset();
     return TRUE;
@@ -855,8 +972,9 @@ bool Workspace::startWalkThroughDesktops( int mode )
     if( !establishTabBoxGrab())
         return false;
     control_grab = TRUE;
-    keys->setEnabled( false );
-    client_keys->setEnabled( false );
+    keys->suspend( true );
+    disable_shortcuts_keys->suspend( true );
+    client_keys->suspend( true );
     tab_box->setMode( (TabBox::Mode) mode );
     tab_box->reset();
     return TRUE;
@@ -886,40 +1004,57 @@ void Workspace::walkThroughDesktops( bool forward )
 
 void Workspace::CDEWalkThroughWindows( bool forward )
     {
-    Client* c = topClientOnDesktop( currentDesktop());
+    Client* c = NULL;
+// this function find the first suitable client for unreasonable focus
+// policies - the topmost one, with some exceptions (can't be keepabove/below,
+// otherwise it gets stuck on them)
+    Q_ASSERT( block_stacking_updates == 0 );
+    for( ClientList::ConstIterator it = stacking_order.fromLast();
+         it != stacking_order.end();
+         --it )
+        {
+        if ( (*it)->isOnCurrentDesktop() && !(*it)->isSpecialWindow()
+            && (*it)->isShown( false ) && (*it)->wantsTabFocus()
+            && !(*it)->keepAbove() && !(*it)->keepBelow())
+            {
+            c = *it;
+            break;
+            }
+        }
     Client* nc = c;
     bool options_traverse_all;
         {
         KConfigGroupSaver saver( KGlobal::config(), "TabBox" );
-        options_traverse_all = KGlobal::config()->readNumEntry("TraverseAll", false );
+        options_traverse_all = KGlobal::config()->readBoolEntry("TraverseAll", false );
         }
 
-    if ( !forward )
+    Client* firstClient = 0;
+    do
         {
-        do
+        nc = forward ? nextStaticClient(nc) : previousStaticClient(nc);
+        if (!firstClient)
             {
-            nc = previousStaticClient(nc);
-            } while (nc && nc != c &&
-        (( !options_traverse_all && !nc->isOnDesktop(currentDesktop())) ||
-         nc->isMinimized() || !nc->wantsTabFocus() ) );
-        }
-    else
-        {
-            do
+            // When we see our first client for the second time,
+            // it's time to stop.
+            firstClient = nc;
+            }
+        else if (nc == firstClient)
             {
-            nc = nextStaticClient(nc);
-            } while (nc && nc != c &&
-                    (( !options_traverse_all && !nc->isOnDesktop(currentDesktop())) ||
-                     nc->isMinimized() || !nc->wantsTabFocus() ) );
-        }
-    if (c && c != nc)
-        lowerClient( c );
+            // No candidates found.
+            nc = 0;
+            break;
+            }
+        } while (nc && nc != c &&
+            (( !options_traverse_all && !nc->isOnDesktop(currentDesktop())) ||
+             nc->isMinimized() || !nc->wantsTabFocus() || nc->keepAbove() || nc->keepBelow() ) );
     if (nc)
         {
+        if (c && c != nc)
+            lowerClient( c );
         if ( options->focusPolicyIsReasonable() )
             {
             activateClient( nc );
-            if( nc->isShade())
+            if( nc->isShade() && options->shadeHover )
                 nc->setShade( ShadeActivated );
             }
         else
@@ -939,7 +1074,7 @@ void Workspace::KDEOneStepThroughWindows( bool forward )
     if( Client* c = tab_box->currentClient() )
         {
         activateClient( c );
-        if( c->isShade())
+        if( c->isShade() && options->shadeHover )
             c->setShade( ShadeActivated );
         }
     }
@@ -1007,8 +1142,9 @@ void Workspace::closeTabBox()
     {
     removeTabBoxGrab();
     tab_box->hide();
-    keys->setEnabled( true );
-    client_keys->setEnabled( true );
+    keys->suspend( false );
+    disable_shortcuts_keys->suspend( false );
+    client_keys->suspend( false );
     tab_grab = FALSE;
     control_grab = FALSE;
     }
@@ -1055,13 +1191,14 @@ void Workspace::tabBoxKeyRelease( const XKeyEvent& ev )
         {
         removeTabBoxGrab();
         tab_box->hide();
-        keys->setEnabled( true );
-        client_keys->setEnabled( true );
+        keys->suspend( false );
+        disable_shortcuts_keys->suspend( false );
+        client_keys->suspend( false );
         tab_grab = false;
         if( Client* c = tab_box->currentClient())
             {
             activateClient( c );
-            if( c->isShade())
+            if( c->isShade() && options->shadeHover )
                 c->setShade( ShadeActivated );
             }
         }
@@ -1069,13 +1206,13 @@ void Workspace::tabBoxKeyRelease( const XKeyEvent& ev )
         {
         removeTabBoxGrab();
         tab_box->hide();
-        keys->setEnabled( true );
-        client_keys->setEnabled( true );
+        keys->suspend( false );
+        disable_shortcuts_keys->suspend( false );
+        client_keys->suspend( false );
         control_grab = False;
         if ( tab_box->currentDesktop() != -1 )
             {
             setCurrentDesktop( tab_box->currentDesktop() );
-                    // popupinfo->showInfo( desktopName(currentDesktop()) ); // AK - not sure
             }
         }
     }
@@ -1109,13 +1246,13 @@ int Workspace::previousDesktopFocusChain( int iDesktop ) const
 */
 Client* Workspace::nextFocusChainClient( Client* c ) const
     {
-    if ( focus_chain.isEmpty() )
+    if ( global_focus_chain.isEmpty() )
         return 0;
-    ClientList::ConstIterator it = focus_chain.find( c );
-    if ( it == focus_chain.end() )
-        return focus_chain.last();
-    if ( it == focus_chain.begin() )
-        return focus_chain.last();
+    ClientList::ConstIterator it = global_focus_chain.find( c );
+    if ( it == global_focus_chain.end() )
+        return global_focus_chain.last();
+    if ( it == global_focus_chain.begin() )
+        return global_focus_chain.last();
     --it;
     return *it;
     }
@@ -1126,14 +1263,14 @@ Client* Workspace::nextFocusChainClient( Client* c ) const
 */
 Client* Workspace::previousFocusChainClient( Client* c ) const
     {
-    if ( focus_chain.isEmpty() )
+    if ( global_focus_chain.isEmpty() )
         return 0;
-    ClientList::ConstIterator it = focus_chain.find( c );
-    if ( it == focus_chain.end() )
-        return focus_chain.first();
+    ClientList::ConstIterator it = global_focus_chain.find( c );
+    if ( it == global_focus_chain.end() )
+        return global_focus_chain.first();
     ++it;
-    if ( it == focus_chain.end() )
-        return focus_chain.first();
+    if ( it == global_focus_chain.end() )
+        return global_focus_chain.first();
     return *it;
     }
 
