@@ -575,6 +575,9 @@ void KonqPopupMenu::setup(KonqPopupFlags kpf)
     if ( isCurrentTrash )
     {
         act = new KAction( i18n( "&Empty Trash Bin" ), "emptytrash", 0, this, SLOT( slotPopupEmptyTrashBin() ), &m_ownActions, "empytrash" );
+        KSimpleConfig trashConfig( "trashrc", true );
+        trashConfig.setGroup( "Status" );
+        act->setEnabled( !trashConfig.readBoolEntry( "Empty", true ) );
         addAction( act );
     }
     addGroup( "editactions" );
@@ -658,145 +661,144 @@ void KonqPopupMenu::setup(KonqPopupFlags kpf)
             }
         }
 
-        QStringList dirs = KGlobal::dirs()->findDirs( "data", "konqueror/servicemenus/" );
-        QStringList::ConstIterator dIt = dirs.begin();
-        QStringList::ConstIterator dEnd = dirs.end();
-
-        for (; dIt != dEnd; ++dIt )
+        // findAllResources() also removes duplicates
+        const QStringList entries = KGlobal::dirs()->findAllResources("data",
+                                                                      "konqueror/servicemenus/*.desktop",
+                                                                      false /* recursive */,
+                                                                      true /* unique */);
+        QStringList::ConstIterator eIt = entries.begin();
+        const QStringList::ConstIterator eEnd = entries.end();
+        for (; eIt != eEnd; ++eIt )
         {
-            QDir dir( *dIt );
+            KSimpleConfig cfg( *eIt, true );
+            cfg.setDesktopGroup();
 
-            QStringList entries = dir.entryList( "*.desktop", QDir::Files );
-            QStringList::ConstIterator eIt = entries.begin();
-            QStringList::ConstIterator eEnd = entries.end();
-
-            for (; eIt != eEnd; ++eIt )
+            if (!KIOSKAuthorizedAction(cfg))
             {
-                KSimpleConfig cfg( *dIt + *eIt, true );
-                cfg.setDesktopGroup();
+                continue;
+            }
 
-                if (!KIOSKAuthorizedAction(cfg))
-                {
+            if ( cfg.hasKey( "X-KDE-ShowIfRunning" ) )
+            {
+                const QString app = cfg.readEntry( "X-KDE-ShowIfRunning" );
+                if ( !kapp->dcopClient()->isApplicationRegistered( app.utf8() ) )
                     continue;
+            }
+            if ( cfg.hasKey( "X-KDE-ShowIfDcopCall" ) )
+            {
+                QString dcopcall = cfg.readEntry( "X-KDE-ShowIfDcopCall" );
+                const QCString app = dcopcall.section(' ', 0,0).utf8();
+
+                //if( !kapp->dcopClient()->isApplicationRegistered( app ))
+                //	continue; //app does not exist so cannot send call
+
+                QByteArray dataToSend;
+                QDataStream dataStream(dataToSend, IO_WriteOnly);
+                dataStream << m_lstPopupURLs;
+
+                QCString replyType;
+                QByteArray replyData;
+                QCString object =    dcopcall.section(' ', 1,-2).utf8();
+                QString function =  dcopcall.section(' ', -1);
+                if(!function.endsWith("(KURL::List)")) {
+                    kdWarning() << "Desktop file " << *eIt << " contains an invalid X-KDE-ShowIfDcopCall - the function must take the exact parameter (KURL::List) and must be specified." << endl;
+                    continue; //Be safe.
                 }
 
-                if ( cfg.hasKey( "X-KDE-ShowIfRunning" ) )
-                {
-                    const QString app = cfg.readEntry( "X-KDE-ShowIfRunning" );
-                    if ( !kapp->dcopClient()->isApplicationRegistered( app.utf8() ) )
-                        continue;
-                }
-		if ( cfg.hasKey( "X-KDE-ShowIfDcopCall" ) )
-		{
-		    QString dcopcall = cfg.readEntry( "X-KDE-ShowIfDcopCall" );
-		    const QCString app = dcopcall.section(' ', 0,0).utf8();
-
-		    //if( !kapp->dcopClient()->isApplicationRegistered( app ))
-		    //	continue; //app does not exist so cannot send call
-
-		    QByteArray dataToSend;
-		    QDataStream dataStream(dataToSend, IO_WriteOnly);
-		    dataStream << m_lstPopupURLs;
-
-		    QCString replyType;
-		    QByteArray replyData;
-		    QCString object =    dcopcall.section(' ', 1,-2).utf8();
-		    QString function =  dcopcall.section(' ', -1);
-		    if(!function.endsWith("(KURL::List)")) {
-			kdWarning() << "Desktop file " << *eIt << " contains an invalid X-KDE-ShowIfDcopCall - the function must take the exact parameter (KURL::List) and must be specified." << endl;
-			continue; //Be safe.
-		    }
-
-		    if(!kapp->dcopClient()->call( app, object,
-				    function.utf8(),
-				    dataToSend, replyType, replyData, true, 1000))
-			continue;
-		    if(replyType != "bool" || !replyData[0])
-			continue;
-
-		}
-                if ( cfg.hasKey( "X-KDE-Protocol" ) )
-                {
-                    const QString protocol = cfg.readEntry( "X-KDE-Protocol" );
-                    if ( protocol != urlForServiceMenu.protocol() )
-                        continue;
-                }
-                else if ( urlForServiceMenu.protocol() == "trash" || urlForServiceMenu.url().startsWith( "system:/trash" ) )
-                {
-                    // Require servicemenus for the trash to ask for protocol=trash explicitely.
-                    // Trashed files aren't supposed to be available for actions.
-                    // One might want a servicemenu for trash.desktop itself though.
+                if(!kapp->dcopClient()->call( app, object,
+                                              function.utf8(),
+                                              dataToSend, replyType, replyData, true, 1000))
+                    continue;
+                if(replyType != "bool" || !replyData[0])
                     continue;
-                }
 
-                if ( cfg.hasKey( "X-KDE-Require" ) )
-                {
-                    const QStringList capabilities = cfg.readListEntry( "X-KDE-Require" );
-                    if ( capabilities.contains( "Write" ) && !sWriting )
-                        continue;
-                }
-                if ( (cfg.hasKey( "Actions" ) || cfg.hasKey( "X-KDE-GetActionMenu") ) && cfg.hasKey( "ServiceTypes" ) )
-                {
-                    const QStringList types = cfg.readListEntry( "ServiceTypes" );
-                    const QStringList excludeTypes = cfg.readListEntry( "ExcludeServiceTypes" );
-                    bool ok = false;
+            }
+            if ( cfg.hasKey( "X-KDE-Protocol" ) )
+            {
+                const QString protocol = cfg.readEntry( "X-KDE-Protocol" );
+                if ( protocol != urlForServiceMenu.protocol() )
+                    continue;
+            }
+            else if ( cfg.hasKey( "X-KDE-Protocols" ) )
+            {
+                QStringList protocols = QStringList::split( "," , cfg.readEntry( "X-KDE-Protocols" ) );
+                if ( !protocols.contains( urlForServiceMenu.protocol() ) )
+                    continue;
+            }
+            else if ( urlForServiceMenu.protocol() == "trash" || urlForServiceMenu.url().startsWith( "system:/trash" ) )
+            {
+                // Require servicemenus for the trash to ask for protocol=trash explicitely.
+                // Trashed files aren't supposed to be available for actions.
+                // One might want a servicemenu for trash.desktop itself though.
+                continue;
+            }
 
-                    // check for exact matches or a typeglob'd mimetype if we have a mimetype
-                    for (QStringList::ConstIterator it = types.begin();
-                         it != types.end() && !ok;
-                         ++it)
+            if ( cfg.hasKey( "X-KDE-Require" ) )
+            {
+                const QStringList capabilities = cfg.readListEntry( "X-KDE-Require" );
+                if ( capabilities.contains( "Write" ) && !sWriting )
+                    continue;
+            }
+            if ( (cfg.hasKey( "Actions" ) || cfg.hasKey( "X-KDE-GetActionMenu") ) && cfg.hasKey( "ServiceTypes" ) )
+            {
+                const QStringList types = cfg.readListEntry( "ServiceTypes" );
+                const QStringList excludeTypes = cfg.readListEntry( "ExcludeServiceTypes" );
+                bool ok = false;
+
+                // check for exact matches or a typeglob'd mimetype if we have a mimetype
+                for (QStringList::ConstIterator it = types.begin();
+                     it != types.end() && !ok;
+                     ++it)
+                {
+                    // first check if we have an all mimetype
+                    bool checkTheMimetypes = false;
+                    if (*it == "all/all" ||
+                        *it == "allfiles" /*compat with KDE up to 3.0.3*/)
                     {
-                        // first check if we have an all mimetype
-                        bool checkTheMimetypes = false;
-                        if (*it == "all/all" ||
-                            *it == "allfiles" /*compat with KDE up to 3.0.3*/)
-                        {
-                            checkTheMimetypes = true;
-                        }
+                        checkTheMimetypes = true;
+                    }
 
-                        // next, do we match all files?
-                        if (!ok &&
-                            !isDirectory &&
-                            *it == "all/allfiles")
-                        {
-                            checkTheMimetypes = true;
-                        }
+                    // next, do we match all files?
+                    if (!ok &&
+                        !isDirectory &&
+                        *it == "all/allfiles")
+                    {
+                        checkTheMimetypes = true;
+                    }
 
-                        // if we have a mimetype, see if we have an exact or a type globbed match
-                        if (!ok &&
-                            (!m_sMimeType.isEmpty() &&
-                              *it == m_sMimeType) ||
-                            (!mimeGroup.isEmpty() &&
-                             ((*it).right(1) == "*" &&
-                              (*it).left((*it).find('/')) == mimeGroup)))
-                        {
-                            checkTheMimetypes = true;
-                        }
+                    // if we have a mimetype, see if we have an exact or a type globbed match
+                    if (!ok &&
+                        (!m_sMimeType.isEmpty() &&
+                         *it == m_sMimeType) ||
+                        (!mimeGroup.isEmpty() &&
+                         ((*it).right(1) == "*" &&
+                          (*it).left((*it).find('/')) == mimeGroup)))
+                    {
+                        checkTheMimetypes = true;
+                    }
 
-                        if (checkTheMimetypes)
+                    if (checkTheMimetypes)
+                    {
+                        ok = true;
+                        for (QStringList::ConstIterator itex = excludeTypes.begin(); itex != excludeTypes.end(); ++itex)
                         {
-                            ok = true;
-                            for (QStringList::ConstIterator itex = excludeTypes.begin(); itex != excludeTypes.end(); ++itex)
+                            if( ((*itex).right(1) == "*" && (*itex).left((*itex).find('/')) == mimeGroup) ||
+                                ((*itex) == m_sMimeType) )
                             {
-                                if( ((*itex).right(1) == "*" && (*itex).left((*itex).find('/')) == mimeGroup) ||
-                                    ((*itex) == m_sMimeType) )
-                                {
-                                    ok = false;
-                                    break;
-                                }
+                                ok = false;
+                                break;
                             }
                         }
                     }
+                }
 
-                    if ( ok )
-                    {
-                        const QString priority = cfg.readEntry("X-KDE-Priority");
-                        const QString submenuName = cfg.readEntry( "X-KDE-Submenu" );
-
-                        ServiceList* list = s.selectList( priority, submenuName );
-                        (*list) += KDEDesktopMimeType::userDefinedServices( *dIt + *eIt, cfg, url.isLocalFile(), m_lstPopupURLs );
-		    }
+                if ( ok )
+                {
+                    const QString priority = cfg.readEntry("X-KDE-Priority");
+                    const QString submenuName = cfg.readEntry( "X-KDE-Submenu" );
 
+                    ServiceList* list = s.selectList( priority, submenuName );
+                    (*list) += KDEDesktopMimeType::userDefinedServices( *eIt, cfg, url.isLocalFile(), m_lstPopupURLs );
                 }
             }
         }
@@ -852,13 +854,29 @@ void KonqPopupMenu::setup(KonqPopupFlags kpf)
                 KTrader::OfferList::ConstIterator it = offers.begin();
                 for( ; it != offers.end(); it++ )
                 {
-                    if ((*it)->noDisplay())
-                        continue;
+                    KService::Ptr service = (*it);
+
+                    // Skip OnlyShowIn=Foo and NotShowIn=KDE entries,
+                    // but still offer NoDisplay=true entries, that's the
+                    // whole point of such desktop files. This is why we don't
+                    // use service->noDisplay() here.
+                    const QString onlyShowIn = service->property("OnlyShowIn", QVariant::String).toString();
+                    if ( !onlyShowIn.isEmpty() ) {
+                        const QStringList aList = QStringList::split(';', onlyShowIn);
+                        if (!aList.contains("KDE"))
+                            continue;
+                    }
+                    const QString notShowIn = service->property("NotShowIn", QVariant::String).toString();
+                    if ( !notShowIn.isEmpty() ) {
+                        const QStringList aList = QStringList::split(';', notShowIn);
+                        if (aList.contains("KDE"))
+                            continue;
+                    }
 
                     QCString nam;
                     nam.setNum( id );
 
-                    QString actionName( (*it)->name() );
+                    QString actionName( (*it)->name().replace("&", "&&") );
                     if ( menu == m_menuElement ) // no submenu -> prefix single offer
                         actionName = i18n( "Open with %1" ).arg( actionName );
 
